/* OGMRip - A library for dvd video ripping and encoding
 * AudioCopyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a audio_copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-acopy.h"
#include "ogmrip-backend.h"

#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

#include <stdio.h>

static gint ogmrip_audio_copy_run (OGMSpSpawn *spawn);

G_DEFINE_TYPE (OGMRipAudioCopy, ogmrip_audio_copy, OGMRIP_TYPE_AUDIO)

static void
ogmrip_audio_copy_class_init (OGMRipAudioCopyClass *klass)
{
  OGMSpSpawnClass *spawn_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);

  spawn_class->run = ogmrip_audio_copy_run;
}

static void
ogmrip_audio_copy_init (OGMRipAudioCopy *audio_copy)
{
}

static gint
ogmrip_audio_copy_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gchar **argv;
  gint result;

  argv = ogmrip_backend_acopy_command (OGMRIP_AUDIO (spawn), NULL, NULL);
  if (!argv)
    return OGMSP_RESULT_ERROR;

  child = ogmsp_exec_newv (argv);
  ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_acopy_watch, spawn, TRUE, FALSE, FALSE);
  ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
  g_object_unref (child);

  result = OGMSP_SPAWN_CLASS (ogmrip_audio_copy_parent_class)->run (spawn);

  ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);

  return result;
}

OGMSpSpawn *
ogmrip_audio_copy_new (OGMDvdAudioStream *audio, const gchar *output)
{
  OGMSpSpawn *spawn;

  spawn = g_object_new (OGMRIP_TYPE_AUDIO_COPY, NULL);
  ogmrip_codec_set_output (OGMRIP_CODEC (spawn), output);
  ogmrip_audio_set_dvd_audio_stream (OGMRIP_AUDIO (spawn), audio);

  return spawn;
}

