/* OGMRip - A library for dvd video ripping and encoding
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef __OGMRIP_BACKEND_H__
#define __OGMRIP_BACKEND_H__

#include <ogmsp-exec.h>
#include <ogmrip-audio.h>
#include <ogmrip-video.h>
#include <ogmrip-misc.h>
#include <ogmrip-subp.h>
#include <ogmrip-mux.h>

G_BEGIN_DECLS

gdouble  ogmrip_backend_wav_watch         (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipAudio *audio);
gdouble  ogmrip_backend_acopy_watch       (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipAudio *audio);
gdouble  ogmrip_backend_xvid_watch        (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipVideo *video);
gdouble  ogmrip_backend_lavc_watch        (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipVideo *video);
gdouble  ogmrip_backend_x264_watch        (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipVideo *video);
gdouble  ogmrip_backend_yuv4mpeg_watch    (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipVideo *video);
gdouble  ogmrip_backend_crop_watch        (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipVideo *video);
gdouble  ogmrip_backend_merge_watch       (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipMux   *merge);
gdouble  ogmrip_backend_split_watch       (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipMisc  *split);
gdouble  ogmrip_backend_matroska_watch    (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipMux   *matroska);
gdouble  ogmrip_backend_avi_watch         (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipMux   *avi);
gdouble  ogmrip_backend_vobsub_watch      (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipSubp  *subp);
gdouble  ogmrip_backend_pgm_watch         (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipSubp  *subp);
gdouble  ogmrip_backend_ocr_watch         (OGMSpExec   *exec, 
                                           const gchar *buffer, 
                                           OGMRipSubp  *subp);

gchar ** ogmrip_backend_wav_command       (OGMRipAudio *audio,
                                           gboolean    header,
                                           const gchar *input,
                                           const gchar *output);
gchar ** ogmrip_backend_acopy_command     (OGMRipAudio *audio,
                                           const gchar *input,
                                           const gchar *output);
gchar ** ogmrip_backend_vorbis_command    (OGMRipAudio *audio,
                                           const gchar *input,
                                           const gchar *output);
gchar ** ogmrip_backend_mp3_command       (OGMRipAudio *audio,
                                           const gchar *input,
                                           const gchar *output);
gchar ** ogmrip_backend_xvid_command      (OGMRipVideo *video,
                                           const gchar *input,
                                           const gchar *output,
                                           const gchar *log);
gchar ** ogmrip_backend_lavc_command      (OGMRipVideo *video,
                                           const gchar *input,
                                           const gchar *output, 
                                           const gchar *log);
gchar ** ogmrip_backend_x264_command      (OGMRipVideo *video,
                                           const gchar *input,
                                           const gchar *output, 
                                           const gchar *log);
gchar ** ogmrip_backend_yuv4mpeg_command  (OGMRipVideo *video,
                                           const gchar *input,
                                           const gchar *output, 
                                           const gchar *log);
gchar ** ogmrip_backend_theora_command    (OGMRipVideo *video,
                                           const gchar *input,
                                           const gchar *output, 
                                           const gchar *log);
gchar ** ogmrip_backend_crop_command      (OGMRipVideo *video,
                                           const gchar *input,
                                           const gchar *output);

gchar ** ogmrip_backend_vobsub_command    (OGMRipSubp   *subp,
                                           const gchar  *input,
                                           const gchar  *output);
gchar ** ogmrip_backend_pgm_command       (OGMRipSubp   *subp,
                                           const gchar  *input,
                                           const gchar  *output);
gchar ** ogmrip_backend_ocr_command       (OGMRipSubp   *subp,
                                           const gchar  *input,
                                           const gchar  *output);
gchar ** ogmrip_backend_srt_command       (OGMRipSubp   *subp,
                                           const gchar  *input,
                                           const gchar  *output);

gchar ** ogmrip_backend_split_command     (OGMRipMisc  *split);
gchar ** ogmrip_backend_merge_command     (OGMRipMux   *merge);
gchar ** ogmrip_backend_matroska_command  (OGMRipMux   *matroska);
gchar ** ogmrip_backend_avi_command       (OGMRipMux   *avi);

G_END_DECLS

#endif /* __OGMRIP_BACKEND_H__ */

