/* OGMRip - A library for dvd video ripping and encoding
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-codec.h"

#define OGMRIP_CODEC_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_CODEC, OGMRipCodecPriv))

struct _OGMRipCodecPriv
{
  OGMDvdVideoStream *stream;
  gchar *output;

  gboolean dirty;
  gulong length;

  OGMDvdTime time_;

  guint start;
  gint end;
};

static void ogmrip_codec_finalize (GObject *gobject);

G_DEFINE_ABSTRACT_TYPE (OGMRipCodec, ogmrip_codec, OGMSP_TYPE_BIN)

static void
ogmrip_codec_class_init (OGMRipCodecClass *klass)
{
  GObjectClass *gobject_class;

  gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->finalize = ogmrip_codec_finalize;

  g_type_class_add_private (klass, sizeof (OGMRipCodecPriv));
}

static void
ogmrip_codec_init (OGMRipCodec *codec)
{
  codec->priv = OGMRIP_CODEC_GET_PRIVATE (codec);
  codec->priv->start = 0;
  codec->priv->end = -1;
}

static void
ogmrip_codec_finalize (GObject *gobject)
{
  OGMRipCodec *codec;

  codec = OGMRIP_CODEC (gobject);

  g_free (codec->priv->output);

  if (codec->priv->stream)
    ogmdvd_video_stream_unref (codec->priv->stream);

  G_OBJECT_CLASS (ogmrip_codec_parent_class)->finalize (gobject);
}

void
ogmrip_codec_set_output (OGMRipCodec *codec, const gchar *output)
{
  g_return_if_fail (OGMRIP_IS_CODEC (codec));

  g_free (codec->priv->output);
  codec->priv->output = g_strdup (output);
}

G_CONST_RETURN gchar *
ogmrip_codec_get_output (OGMRipCodec *codec)
{
  g_return_val_if_fail (OGMRIP_IS_CODEC (codec), NULL);

  return codec->priv->output;
}

void
ogmrip_codec_set_input (OGMRipCodec *codec, OGMDvdVideoStream *stream)
{
  g_return_if_fail (OGMRIP_IS_CODEC (codec));
  g_return_if_fail (stream != NULL);

  ogmdvd_video_stream_ref (stream);

  if (codec->priv->stream)
    ogmdvd_video_stream_unref (codec->priv->stream);

  codec->priv->stream = stream;
  codec->priv->dirty = TRUE;
}

OGMDvdVideoStream *
ogmrip_codec_get_input (OGMRipCodec *codec)
{
  g_return_val_if_fail (OGMRIP_IS_CODEC (codec), NULL);

  return codec->priv->stream;
}

void
ogmrip_codec_set_chapters (OGMRipCodec *codec, guint start, guint end)
{
  g_return_if_fail (OGMRIP_IS_CODEC (codec));

  codec->priv->dirty = TRUE;
  codec->priv->start = MIN (start, end);
  codec->priv->end = MAX (start, end);
}

void
ogmrip_codec_get_chapters (OGMRipCodec *codec, guint *start, guint *end)
{
  g_return_if_fail (OGMRIP_IS_CODEC (codec));
  g_return_if_fail (start != NULL);
  g_return_if_fail (end != NULL);

  *start = codec->priv->start;
  *end = codec->priv->end;
}

glong
ogmrip_codec_get_length (OGMRipCodec *codec, OGMDvdTime *time_)
{
  g_return_val_if_fail (OGMRIP_IS_CODEC (codec), -1);

  if (!codec->priv->stream)
    return -1;

  if (codec->priv->dirty)
  {
    if (codec->priv->start == 0 && codec->priv->end == -1)
      codec->priv->length = ogmdvd_video_stream_get_length (codec->priv->stream, &codec->priv->time_);
    else
      codec->priv->length = ogmdvd_video_stream_get_chapters_length (codec->priv->stream, 
            &codec->priv->time_, codec->priv->start, codec->priv->end);

    codec->priv->dirty = FALSE;
  }

  if (time_)
    *time_ = codec->priv->time_;

  return codec->priv->length;
}

