/* OGMRip - A library for dvd video ripping and encoding
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-backend.h"
#include "ogmrip-merge.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

static gint ogmrip_merge_run (OGMSpSpawn *spawn);

G_DEFINE_TYPE (OGMRipMerge, ogmrip_merge, OGMRIP_TYPE_MUX)

static void
ogmrip_merge_class_init (OGMRipMergeClass *klass)
{
  OGMSpSpawnClass *spawn_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);

  spawn_class->run = ogmrip_merge_run;
}

static void
ogmrip_merge_init (OGMRipMerge *merge)
{
}

static gint
ogmrip_merge_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gchar **argv;
  gint result;

  argv = ogmrip_backend_merge_command (OGMRIP_MUX (spawn));
  if (!argv)
    return OGMSP_RESULT_ERROR;

  child = ogmsp_exec_newv (argv);
  ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_merge_watch, spawn, TRUE, FALSE, FALSE);
  ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
  g_object_unref (child);

  result = OGMSP_SPAWN_CLASS (ogmrip_merge_parent_class)->run (spawn);

  ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);

  return result;
}

OGMSpSpawn *
ogmrip_merge_new (const gchar *output)
{
  OGMRipMerge *merge;

  g_return_val_if_fail (output && *output, NULL);

  merge = g_object_new (OGMRIP_TYPE_MERGE, NULL);
  ogmrip_mux_set_output (OGMRIP_MUX (merge), output);

  return OGMSP_SPAWN (merge);
}

