/* OGMRip - A library for dvd video ripping and encoding
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-misc.h"

#define OGMRIP_MISC_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_MISC, OGMRipMiscPriv))

struct _OGMRipMiscPriv
{
  gchar *input;
  gchar *output;
};

static void ogmrip_misc_finalize (GObject    *gobject);

G_DEFINE_TYPE (OGMRipMisc, ogmrip_misc, OGMSP_TYPE_BIN)

static void
ogmrip_misc_class_init (OGMRipMiscClass *klass)
{
  GObjectClass *gobject_class;

  gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->finalize = ogmrip_misc_finalize;

  g_type_class_add_private (klass, sizeof (OGMRipMiscPriv));
}

static void
ogmrip_misc_init (OGMRipMisc *misc)
{
  misc->priv = OGMRIP_MISC_GET_PRIVATE (misc);
}

static void
ogmrip_misc_finalize (GObject *gobject)
{
  OGMRipMisc *misc;

  misc = OGMRIP_MISC (gobject);

  g_free (misc->priv->input);
  g_free (misc->priv->output);

  G_OBJECT_CLASS (ogmrip_misc_parent_class)->finalize (gobject);
}

G_CONST_RETURN gchar *
ogmrip_misc_get_input (OGMRipMisc *misc)
{
  g_return_val_if_fail (OGMRIP_IS_MISC (misc), NULL);

  return misc->priv->input;
}

void
ogmrip_misc_set_input (OGMRipMisc *misc, const gchar *input)
{
  g_return_if_fail (OGMRIP_IS_MISC (misc));
  g_return_if_fail (input && *input);

  g_free (misc->priv->input);
  misc->priv->input = g_strdup (input);
}

G_CONST_RETURN gchar *
ogmrip_misc_get_output (OGMRipMisc *misc)
{
  g_return_val_if_fail (OGMRIP_IS_MISC (misc), NULL);

  return misc->priv->output;
}

void
ogmrip_misc_set_output (OGMRipMisc *misc, const gchar *output)
{
  g_return_if_fail (OGMRIP_IS_MISC (misc));
  g_return_if_fail (output && *output);

  g_free (misc->priv->output);
  misc->priv->output = g_strdup (output);
}

