/* OGMRip - A library for dvd video ripping and encoding
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-backend.h"
#include "ogmrip-split.h"
#include "ogmsp-exec.h"

#define OGMRIP_SPLIT_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_SPLIT, OGMRipSplitPriv))

struct _OGMRipSplitPriv
{
  guint tsize;
};

static gint ogmrip_split_run      (OGMSpSpawn *spawn);

G_DEFINE_TYPE (OGMRipSplit, ogmrip_split, OGMRIP_TYPE_MISC)

static void
ogmrip_split_class_init (OGMRipSplitClass *klass)
{
  OGMSpSpawnClass *spawn_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);

  spawn_class->run = ogmrip_split_run;

  g_type_class_add_private (klass, sizeof (OGMRipSplitPriv));
}

static void
ogmrip_split_init (OGMRipSplit *split)
{
  split->priv = OGMRIP_SPLIT_GET_PRIVATE (split);
}

static gint
ogmrip_split_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gchar **argv;
  gint result;

  argv = ogmrip_backend_split_command (OGMRIP_MISC (spawn));

  child = ogmsp_exec_newv (argv);
  ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_split_watch, spawn, TRUE, FALSE, FALSE);
  ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
  g_object_unref (child);

  result = OGMSP_SPAWN_CLASS (ogmrip_split_parent_class)->run (spawn);

  ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);

  return result;
}

OGMSpSpawn *
ogmrip_split_new (const gchar *input, const gchar *output, guint tsize)
{
  OGMRipSplit *split;

  g_return_val_if_fail (input && *input, NULL);
  g_return_val_if_fail (output && *output, NULL);
  g_return_val_if_fail (tsize > 0, NULL);

  split = g_object_new (OGMRIP_TYPE_SPLIT, NULL);
  split->priv->tsize = tsize;

  ogmrip_misc_set_input (OGMRIP_MISC (split), input);
  ogmrip_misc_set_output (OGMRIP_MISC (split), output);

  return OGMSP_SPAWN (split);
}

gint
ogmrip_split_get_tsize (OGMRipSplit *split)
{
  g_return_val_if_fail (OGMRIP_IS_SPLIT (split), -1);

  return split->priv->tsize;
}

