/* OGMRip - A library for dvd video ripping and encoding
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-srt.h"
#include "ogmrip-backend.h"
#include "ogmrip-tools.h"

#include "ogmsp-queue.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

#include <stdio.h>
#include <string.h>
#include <unistd.h>

static gint ogmrip_srt_run (OGMSpSpawn *spawn);

G_DEFINE_TYPE (OGMRipSrt, ogmrip_srt, OGMRIP_TYPE_SUBP)

static void
ogmrip_srt_class_init (OGMRipSrtClass *klass)
{
  OGMSpSpawnClass *spawn_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);

  spawn_class->run = ogmrip_srt_run;
}

static void
ogmrip_srt_init (OGMRipSrt *srt)
{
}

static gint
ogmrip_srt_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gint result, fd;
  gchar **argv;

  GDir *dir;
  GPatternSpec *pspec;
  gchar *pattern, *str;
  const gchar *name;

  gchar *tmp_file, *srtx_file;

  result = OGMSP_RESULT_ERROR;

  fd = g_file_open_tmp ("sub.XXXXXX", &tmp_file, NULL);
  if (fd < 0)
    return OGMSP_RESULT_ERROR;
  unlink (tmp_file);
  close (fd);

  srtx_file = g_strconcat (tmp_file, ".srtx", NULL);

  argv = ogmrip_backend_vobsub_command (OGMRIP_SUBP (spawn), NULL, tmp_file);
  if (argv)
  {
    child = ogmsp_exec_newv (argv);
    ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_vobsub_watch, spawn, TRUE, FALSE, FALSE);
    ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
    g_object_unref (child);

    result = OGMSP_SPAWN_CLASS (ogmrip_srt_parent_class)->run (spawn);

    ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);
  }

  if (result == OGMSP_RESULT_COMPLETED)
  {
    result = OGMSP_RESULT_ERROR;

    argv = ogmrip_backend_pgm_command (OGMRIP_SUBP (spawn), tmp_file, NULL);
    if (argv)
    {
      child = ogmsp_exec_newv (argv);
      ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_pgm_watch, spawn, TRUE, FALSE, FALSE);
      result = ogmsp_spawn_run (child);
      g_object_unref (child);
    }
  }

  if (result == OGMSP_RESULT_COMPLETED)
  {
    result = OGMSP_RESULT_ERROR;

    dir = g_dir_open (g_get_tmp_dir (), 0, NULL);
    if (dir)
    {
      gchar *basename;

      basename = g_path_get_basename (tmp_file);
      pattern = g_strconcat (basename, "*.pgm", NULL);
      pspec = g_pattern_spec_new (pattern);
      g_free (basename);
      g_free (pattern);

      while ((name = g_dir_read_name (dir)))
      {
        if (g_pattern_match (pspec, strlen (name), name, NULL))
        {
          str = g_build_filename (g_get_tmp_dir (), name, NULL);
          argv = ogmrip_backend_ocr_command (OGMRIP_SUBP (spawn), str, NULL);

          if (argv)
          {
            child = ogmsp_exec_newv (argv);
            ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_ocr_watch, spawn, FALSE, TRUE, TRUE);
            result = ogmsp_spawn_run (child);
            g_object_unref (child);

            if (result != OGMSP_RESULT_COMPLETED)
              break;
          }

          unlink (str);
          g_free (str);
        }
      }

      g_pattern_spec_free (pspec);
      g_dir_close (dir);
    }
  }

  if (result == OGMSP_RESULT_COMPLETED)
  {
    result = OGMSP_RESULT_ERROR;

    argv = ogmrip_backend_srt_command (OGMRIP_SUBP (spawn), srtx_file, NULL);
    if (argv)
    {
      child = ogmsp_exec_newv (argv);
      result = ogmsp_spawn_run (child);
      g_object_unref (child);
    }
  }

  if (g_file_test (srtx_file, G_FILE_TEST_EXISTS))
    unlink (srtx_file);
  g_free (srtx_file);

  srtx_file = g_strconcat (tmp_file, ".idx", NULL);
  if (g_file_test (srtx_file, G_FILE_TEST_EXISTS))
    unlink (srtx_file);
  g_free (srtx_file);

  srtx_file = g_strconcat (tmp_file, ".sub", NULL);
  if (g_file_test (srtx_file, G_FILE_TEST_EXISTS))
    unlink (srtx_file);
  g_free (srtx_file);

  dir = g_dir_open (g_get_tmp_dir (), 0, NULL);
  if (dir)
  {
    gchar *basename;

    basename = g_path_get_basename (tmp_file);
    pattern = g_strconcat (basename, "*.pgm*", NULL);
    pspec = g_pattern_spec_new (pattern);
    g_free (basename);
    g_free (pattern);

    while ((name = g_dir_read_name (dir)))
    {
      if (g_pattern_match (pspec, strlen (name), name, NULL))
      {
        str = g_build_filename (g_get_tmp_dir (), name, NULL);
        unlink (str);
        g_free (str);
      }
    }

    g_pattern_spec_free (pspec);
    g_dir_close (dir);
  }

  g_free (tmp_file);

  return result;
}

OGMSpSpawn *
ogmrip_srt_new (OGMDvdSubpStream *subp, const gchar *output)
{
  OGMSpSpawn *spawn;

  spawn = g_object_new (OGMRIP_TYPE_SRT, NULL);
  ogmrip_codec_set_output (OGMRIP_CODEC (spawn), output);
  ogmrip_subp_set_dvd_subp_stream (OGMRIP_SUBP (spawn), subp);

  return spawn;
}

