/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef __OGMRIP_HELPER_H__
#define __OGMRIP_HELPER_H__

#include <gtk/gtk.h>
#include <cd-drive.h>
#include <sys/types.h>
#include <gconf/gconf-client.h>

G_BEGIN_DECLS

#define OGMRIP_ERROR ogmrip_error_quark ()

typedef enum
{
  OGMRIP_ERROR_NONE,
  OGMRIP_ERROR_INTERNAL
} OGMRipError;

typedef struct
{
  gchar *name;
  gint fd;
} GFile;

/*
 * Glib
 */
gboolean g_dir_make                      (const gchar    *path, 
                                          mode_t         mode, 
                                          GError         **error);
gint64   g_fs_space_left                 (const gchar    *filename);
gchar *  g_fs_get_mount_point            (const gchar    *filename);

/*
 * GFile
 */
GFile *  g_file_open                     (const gchar    *name,
                                          GError         **error);
GFile *  g_file_open_temp                (const gchar    *tmp, 
                                          GError         **error);
glong    g_file_get_size                 (GFile          *file);
void     g_file_close                    (GFile          **file,
                                          gboolean       do_unlink);
gboolean g_file_rename                   (GFile          *file,
                                          const gchar    *new_name,
                                          GError         **error);

/*
 * GConf
 */
gint     gconf_client_get_int_default    (GConfClient    *client,
                                          const gchar    *key,
                                          gint           def);
gboolean gconf_client_get_bool_default   (GConfClient    *client,
                                          const gchar    *key,
                                          gboolean       def);
gchar *  gconf_client_get_string_default (GConfClient    *client,
                                          const gchar    *key,
                                          const gchar    *def);

/*
 * Gtk+
 */
void     gtk_window_set_parent           (GtkWindow      *window, 
                                          GtkWindow      *parent);
void     gtk_window_set_icon_from_stock  (GtkWindow      *window,
                                          const gchar    *stock_id);
gint     gtk_radio_button_get_active     (GtkRadioButton *radio);
void     gtk_radio_button_set_active     (GtkRadioButton *radio, 
                                          guint          index);
gboolean gtk_tree_model_iter_prev        (GtkTreeModel   *tree_model,
                                          GtkTreeIter    *iter);
void     gtk_label_set_int               (GtkLabel       *label,
                                          gint           value);
gint     gtk_label_get_int               (GtkLabel       *label);

/*
 * OGMDvd
 */
G_CONST_RETURN 
gchar *  ogmdvd_get_default_device       (void);
gboolean ogmdvd_is_valid_drive           (const CDDrive  *drive);
gboolean ogmdvd_is_valid_device          (const gchar    *device);
gboolean ogmdvd_is_valid_media           (const gchar    *device);
gboolean ogmdvd_tray_is_open             (const gchar    *device);
gboolean ogmdvd_tray_open                (const gchar    *device);
G_CONST_RETURN 
gchar *  ogmdvd_check_device             (const gchar    *device);

/*
 * OGMRip
 */
GQuark   ogmrip_error_quark              (void);

gint     ogmrip_message_dialog           (GtkWindow      *parent, 
                                          GtkMessageType type, 
                                          const gchar    *format, 
                                          ...);
G_CONST_RETURN
gchar *  ogmrip_get_tmp_dir              (void);
void     ogmrip_set_tmp_dir              (const gchar *dir);
gint     ogmrip_file_open_tmp            (const gchar *tmpl,
                                          gchar       **name_used,
                                          GError      **error);

G_END_DECLS

#endif /* __OGMRIP_HELPER_H__ */

