/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"
#include "ogmrip-pref.h"
#include "ogmrip-progress.h"
#include "ogmrip-helper.h"
#include "ogmrip-gconf.h"
#include "ogmrip-options.h"
#include <ogmrip-dvd-chooser.h>

#include <ogmdvd.h>
#include <ogmrip-avi.h>
#include <ogmrip-mp3.h>
#include <ogmrip-wav.h>
#include <ogmrip-lavc.h>
#include <ogmrip-xvid.h>
#include <ogmrip-x264.h>
#include <ogmrip-acopy.h>
#include <ogmrip-merge.h>
#include <ogmrip-split.h>
#include <ogmrip-vobsub.h>
#include <ogmrip-vorbis.h>
#include <ogmsp-enums.h>

#ifdef HAVE_MKV_SUPPORT
#include <ogmrip-mkv.h>
#endif

#ifdef HAVE_SRT_SUPPORT
#include <ogmrip-srt.h>
#endif

#ifdef HAVE_ENCHANT_SUPPORT
#include "ogmrip-spell.h"
#endif

#include <gnome.h>
#include <glade/glade.h>
#include <string.h>
#include <fcntl.h>
#include <math.h>

#define OGMRIP_GLADE_FILE "ogmrip/ogmrip-main.glade"
#define OGMRIP_ICON_FILE  "pixmaps/ogmrip.png"

#define OGMRIP_DEFAULT_FILE_NAME "movie"

#define G_FILE(f)        ((GFile *) (f))
#define GTK_BOX_CHILD(b) ((GtkBoxChild *) (b))

enum
{
  EXTRACT,
  CHAPTER,
  TITLE,
  LENGTH,
  FRAMES,
  COL_LAST
};

enum
{
  OGMRIP_CONTAINER_OGM,
  OGMRIP_CONTAINER_AVI,
  OGMRIP_CONTAINER_MKV
};

enum
{
  OGMRIP_AUDIO_VORBIS,
  OGMRIP_AUDIO_MP3,
  OGMRIP_AUDIO_WAV,
  OGMRIP_AUDIO_COPY
};

enum
{
  OGMRIP_VIDEO_XVID,
  OGMRIP_VIDEO_LAVC,
  OGMRIP_VIDEO_X264,
  OGMRIP_VIDEO_THEORA
};

typedef struct
{
  GtkWidget *window;
  GtkWidget *appbar;

  GtkWidget *pref_dialog;
  GtkWidget *options_dialog;
  GtkWidget *progress_dialog;

  GtkWidget *title_entry;
  GtkWidget *video_combo;
  GtkWidget *length_label;
  GtkWidget *relative_check;

#ifdef OGMRIP_DEVEL_MODE
  GtkWidget *audio_vbox;
  GtkWidget *subp_vbox;
#else /* OGMRIP_DEVEL_MODEL */
  GtkWidget *audio_combo;
  GtkWidget *subp_combo;
#endif /* OGMRIP_DEVEL_MODEL */

  GtkListStore *chapters_store;

  GtkAction *extract_action;
  GtkWidget *extract_button;

  OGMDvd *dvd;
  GtkWidget *open_disk_dialog;
  GtkWidget *open_struct_dialog;
  GtkWidget *dvd_chooser;

  GtkActionGroup *action_group;
  GtkUIManager *ui_manager;

  GConfClient *gconf;

  gboolean encoding;

} OGMRipData;

static GtkActionEntry action_entries[] =
{
  { "FileMenu",    NULL,                  N_("_File"),         NULL,          NULL,                           NULL },
  { "Load",        GTK_STOCK_CDROM,       N_("_Load"),         "<ctl>L",      N_("Load a DVD disk"),          NULL },
  { "Open",        GTK_STOCK_OPEN,        NULL,                NULL,          N_("Open a DVD structure"),     NULL },
  { "Extract",     GTK_STOCK_CONVERT,     N_("E_xtract"),      "<ctl>Return", N_("Extract selected streams"), NULL },
  { "Quit",        GTK_STOCK_QUIT,        NULL,                NULL,          N_("Exit OGMRip"),              NULL },
  { "EditMenu",    NULL,                  N_("_Edit"),         NULL,          NULL,                           NULL },
  { "SelectAll",   NULL,                  N_("Select _All"),   "<ctl>A",      N_("Select all chapters"),      NULL },
  { "DeselectAll", NULL,                  N_("_Deselect All"), "<ctl>D",      N_("Deselect all chapters"),    NULL },
  { "Preferences", GTK_STOCK_PREFERENCES, NULL,                NULL,          N_("Edit the preferences"),     NULL },
  { "HelpMenu",    NULL,                  N_("_Help"),         NULL,          NULL,                           NULL },
  { "About",       GNOME_STOCK_ABOUT,     N_("_About"),        NULL,          N_("About OGMRip"),             NULL },
};

static const gchar *ui_description =
"<ui>"
"  <menubar name='Menubar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Load'/>"
"      <menuitem action='Open'/>"
"      <menuitem action='Extract'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='EditMenu'>"
"      <menuitem action='SelectAll'/>"
"      <menuitem action='DeselectAll'/>"
"      <separator/>"
"      <menuitem action='Preferences'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"  <toolbar name='Toolbar'>"
"    <toolitem action='Load'/>"
"    <toolitem action='Open'/>"
"    <separator/>"
"    <toolitem action='Extract'/>"
"  </toolbar>"
"</ui>";

#ifdef OGMRIP_DEVEL_MODE
static void ogmrip_main_add_stream_clicked    (GtkWidget  *button,
                                               OGMRipData *data);
static void ogmrip_main_remove_stream_clicked (GtkWidget  *button,
                                               OGMRipData *data);
#endif /* OGMRIP_DEVEL_MODEL */

#ifdef HAVE_ENCHANT_SUPPORT
gboolean
ogmrip_main_spell_check (OGMRipData *data, GFile *file, const gchar *lang)
{
  gint new_fd = -1;
  gchar *new_file = NULL;
  gchar *text, *corrected;
  gboolean retval = FALSE;

  GtkWidget *checker = NULL;
  GIOChannel *input = NULL, *output = NULL;
  GIOStatus status = G_IO_STATUS_NORMAL;

  checker = ogmrip_spell_new (lang);
  if (!checker)
  {
    ogmrip_message_dialog (GTK_WINDOW (data->window), 
        GTK_MESSAGE_ERROR, "<b>%s %s</b>\n\n%s", 
        _("Could not create dictionary."), 
        _("Spell will not be checked."));
    goto spell_check_cleanup;
  }

  input = g_io_channel_new_file (file->name, "r", NULL);
  if (!input)
    goto spell_check_cleanup;

  new_fd = ogmrip_file_open_tmp ("sub.XXXXXX", &new_file, NULL);
  if (new_fd < 0)
    goto spell_check_cleanup;

  output = g_io_channel_unix_new (new_fd);
  if (!output)
    goto spell_check_cleanup;

  gtk_window_set_parent (GTK_WINDOW (checker), GTK_WINDOW (data->window));
  gtk_widget_show (checker);

  do
  {
    status = g_io_channel_read_line (input, &text, NULL, NULL, NULL);
    if (status == G_IO_STATUS_NORMAL)
    {
      retval = ogmrip_spell_check_text (checker, text, &corrected);
      if (retval)
      {
        do
        {
          status = g_io_channel_write_chars (output, corrected ? corrected : text, -1, NULL, NULL);
        }
        while (status == G_IO_STATUS_AGAIN);

        g_free (corrected);
      }
      g_free (text);
    }
  } 
  while (retval == TRUE && (status == G_IO_STATUS_NORMAL || status == G_IO_STATUS_AGAIN));

  retval &= status == G_IO_STATUS_EOF;

spell_check_cleanup:
  if (checker)
    gtk_widget_destroy (checker);

  if (output)
  {
    g_io_channel_shutdown (output, TRUE, NULL);
    g_io_channel_unref (output);
  }

  if (input)
  {
    g_io_channel_shutdown (input, TRUE, NULL);
    g_io_channel_unref (input);
  }

  if (retval)
  {
    unlink (file->name);
    g_free (file->name);
    close (file->fd);
    file->fd = new_fd;
    file->name = new_file;
  }
  else
  {
    close (new_fd);
    unlink (new_file);
    g_free (new_file);
  }

  return retval;
}
#endif /* HAVE_ENCHANT_SUPPORT */

/*
 * Loads a DVD disk or a DVD structure
 */
static void
ogmrip_main_load (OGMRipData *data, gboolean device, const gchar *path)
{
  OGMDvd *dvd;
  OGMDvdVideoStream *vstream;
  GtkTreeModel *model;

  model = gtk_combo_box_get_model (GTK_COMBO_BOX (data->video_combo));
  gtk_list_store_clear (GTK_LIST_STORE (model));

  gnome_appbar_push (GNOME_APPBAR (data->appbar), 
      _("Retrieving DVD information... please wait."));

  dvd = ogmdvd_open (path);

  if (!dvd)
  {
    if (device)
    {
      if (ogmdvd_tray_open (path))
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
            "<b>%s</b>\n\n%s", _("Could not read the DVD"), _("Tray seems to be opened"));
      else if (ogmdvd_is_valid_media (path))
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
            "<b>%s</b>\n\n%s", _("Could not read the DVD"), _("Device does not contain a valid DVD video"));
      else
        ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
            "<b>%s</b>\n\n%s", _("Could not read the DVD"), _("Unknown error"));
    }
    else
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
          "<b>%s</b>\n\n%s", _("Could not read the DVD"), _("Path does not contain a valid DVD structure"));
    }
  }
  else
  {
    gchar *str, *title = NULL;
    guint vid, nvid, lvid;
    glong length, longest;
    gint format, aspect;

    if (data->dvd)
      ogmdvd_unref (data->dvd);
    data->dvd = dvd;

    nvid = ogmdvd_get_num_video_streams (dvd);

    for (vid = lvid = 0, longest = 0; vid < nvid; vid++)
    {
      vstream = ogmdvd_get_video_stream (dvd, vid);
      format = ogmdvd_video_stream_get_format (vstream);
      aspect = ogmdvd_video_stream_get_display_aspect (vstream);
      length = ogmdvd_video_stream_get_length (vstream, NULL);

      if (length > longest)
      {
        longest = length;
        lvid = vid;
      }

      str = g_strdup_printf ("%s %02d (%s, %s)", _("Title"), vid + 1,
          ogmdvd_get_video_format_name (format),
          ogmdvd_get_video_aspect_name (aspect));
      gtk_combo_box_append_text (GTK_COMBO_BOX (data->video_combo), str);
      g_free (str);

      ogmdvd_video_stream_unref (vstream);
    }

    if (nvid > 0)
    {
      title = ogmdvd_get_title (dvd);
      gtk_combo_box_set_active (GTK_COMBO_BOX (data->video_combo), lvid);
    }

    gtk_widget_set_sensitive (data->video_combo, nvid > 0);
    gtk_widget_set_sensitive (data->title_entry, nvid > 0);
    gtk_widget_set_sensitive (data->extract_button, nvid > 0);
    g_object_set (data->extract_action, "sensitive", nvid > 0, NULL);

    gtk_entry_set_text (GTK_ENTRY (data->title_entry), title ? title : "");
    g_free (title);
  }

  gnome_appbar_pop (GNOME_APPBAR (data->appbar));
}

static void
ogmrip_main_fill_audio_combo (GtkWidget *combo, OGMDvdVideoStream *vstream, gint pref)
{
  OGMDvdAudioStream *astream;
  gint channels, format, lang, type;
  gint aid, naid, selected;
  gchar *str;

  naid = ogmdvd_video_stream_get_num_audio_streams (vstream);

  for (aid = selected = 0; aid < naid; aid++)
  {
    astream = ogmdvd_video_stream_get_audio_stream (vstream, aid);
    channels = ogmdvd_audio_stream_get_channels (astream);
    format = ogmdvd_audio_stream_get_format (astream);
    lang = ogmdvd_audio_stream_get_language (astream);
    type = ogmdvd_audio_stream_get_type (astream);

    if (!selected && lang == pref)
      selected = aid;

    if (type > 0)
      str = g_strdup_printf ("%s %02d: %s (%s, %s, %s)", _("Track"), aid + 1,
          ogmdvd_get_audio_type_name (type), ogmdvd_get_language_name (lang),
          ogmdvd_get_audio_format_name (format), ogmdvd_get_audio_channels_name (channels));
    else
      str = g_strdup_printf ("%s %02d: (%s, %s, %s)", _("Track"), aid + 1,
          ogmdvd_get_language_name (lang), ogmdvd_get_audio_format_name (format), 
          ogmdvd_get_audio_channels_name (channels));

    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
    g_free (str);

    ogmdvd_audio_stream_unref (astream);
  }

  gtk_widget_set_sensitive (combo, naid > 0);
  if (naid > 0)
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), selected);
}

static void
ogmrip_main_fill_subp_combo (GtkWidget *combo, OGMDvdVideoStream *vstream, gint pref)
{
  OGMDvdSubpStream *sstream;
  gint nsid, sid, lang, selected;
  gchar *str;

  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("No subtitle"));

  nsid = ogmdvd_video_stream_get_num_subp_streams (vstream);

  for (sid = selected = 0; sid < nsid; sid++)
  {
    sstream = ogmdvd_video_stream_get_subp_stream (vstream, sid);
    lang = ogmdvd_subp_stream_get_language (sstream);

    if (!selected && lang == pref)
      selected = sid;

    str = g_strdup_printf ("%s %02d (%s)", _("Subtitle"), sid + 1,
        ogmdvd_get_language_name (lang));

    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
    g_free (str);

    ogmdvd_subp_stream_unref (sstream);
  }

#ifdef HAVE_SRT_SUPPORT
  gtk_widget_set_sensitive (combo, nsid > 0);
#endif /* HAVE_SRT_SUPPORT */
  if (nsid > 0)
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), selected);
}

static void
ogmrip_main_fill_chapters_list (GtkListStore *store, OGMDvdVideoStream *vstream)
{
  GtkTreeIter iter;
  OGMDvdTime time_;
  gint chap, nchap;
  glong frames;
  gchar *str;

  nchap = ogmdvd_video_stream_get_num_chapters (vstream);
  for (chap = 0; chap < nchap; chap++)
  {
    gtk_list_store_append (store, &iter);

    str = g_strdup_printf ("%s %02d", _("Chapter"), chap + 1);
    gtk_list_store_set (store, &iter, EXTRACT, TRUE, CHAPTER, chap + 1, TITLE, str, -1);
    g_free (str);

    if ((frames = ogmdvd_video_stream_get_chapters_length (vstream, &time_, chap, chap)) > 0)
    {
      str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
      gtk_list_store_set (store, &iter, LENGTH, str, FRAMES, frames, -1);
      g_free (str);
    }
  }
}

#ifdef OGMRIP_DEVEL_MODE
static void
ogmrip_main_add_stream (OGMRipData *data, GtkWidget *combo, GtkWidget *vbox)
{
  GtkWidget *hbox1, *hbox2;
  GtkWidget *button, *image;
  GtkTooltips *tooltips;

  hbox1 = gtk_hbox_new (FALSE, 6);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
  gtk_widget_show (hbox1);

  gtk_box_pack_start (GTK_BOX (hbox1), combo, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox1), hbox2, FALSE, FALSE, 0);
  gtk_widget_show (hbox2);

  tooltips = gtk_tooltips_new ();

  button = gtk_button_new ();
  gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  gtk_tooltips_set_tip (tooltips, button, _("Add a stream"), NULL);

  g_signal_connect (button, "clicked", 
      G_CALLBACK (ogmrip_main_add_stream_clicked), data);

  image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
  gtk_container_add (GTK_CONTAINER (button), image);
  gtk_widget_show (image);

  button = gtk_button_new ();
  gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 0);

  gtk_tooltips_set_tip (tooltips, button, _("Remove the stream"), NULL);

  g_signal_connect (button, "clicked", 
      G_CALLBACK (ogmrip_main_remove_stream_clicked), data);

  image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_BUTTON);
  gtk_container_add (GTK_CONTAINER (button), image);
  gtk_widget_show (image);
}

static void
ogmrip_main_remove_stream (GtkWidget *hbox)
{
  GtkWidget *combo;
  gint id;

  combo = GTK_BOX_CHILD (GTK_BOX (hbox)->children->data)->widget;
  id = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));

  gtk_container_remove (GTK_CONTAINER (hbox->parent), hbox);
}
#endif /* OGMRIP_DEVEL_MODE */

static gchar *
ogmrip_main_build_filename (OGMRipData *data, const gchar *path,
    guint container, guint vcodec, guint acodec, gint alang)
{
  const gchar *title, *lang;

  static gchar *vcodec_name[] =
  {
    "XviD",
    "Lavc",
    "X264",
    "Theora"
  };

  static gchar *acodec_name[] =
  {
    "Vorbis",
    "Mp3",
    "Wave",
    "Ac3"
  };

  static gchar *extension[] = 
  {
    "ogm",
    "avi",
    "mkv"
  };

  gchar *utf8, *filename, *output;

  title = gtk_entry_get_text (GTK_ENTRY (data->title_entry));
  lang = ogmdvd_get_language_name (alang);

  switch (gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_FILENAME, OGMRIP_DEFAULT_FILENAME))
  {
    case 0:
      utf8 = g_strdup_printf ("%s.%s", 
          title, extension[container]);
      break;
    case 1:
      utf8 = g_strdup_printf ("%s - %s.%s", 
          title, lang, extension[container]);
      break;
    case 2:
      utf8 = g_strdup_printf ("%s - %s - %s.%s", 
          title, lang, vcodec_name[vcodec], extension[container]);
      break;
    case 3:
      utf8 = g_strdup_printf ("%s - %s - %s %s.%s", 
          title, lang, vcodec_name[vcodec], acodec_name[acodec], extension[container]);
      break;
    default:
      utf8 = g_strdup_printf ("%s.%s", 
          OGMRIP_DEFAULT_FILE_NAME, extension[container]);
      break;
  }

  filename = g_filename_from_utf8 (utf8, -1, NULL, NULL, NULL);
  g_free (utf8);

  output = g_build_filename (path, filename, NULL);
  g_free (filename);

  return output;
}

/*
 * Checks if container and codecs are compatible
 */
static gboolean
ogmrip_main_check_container (OGMRipData *data, gint container, gint vcocec, gint acodec)
{
  /*
   * AVI cannot contain vorbis streams
   */
  if (container == OGMRIP_CONTAINER_AVI && acodec == OGMRIP_AUDIO_VORBIS)
  {
    ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, "<b>%s</b>\n\n%s",
        _("The video container and the audio codec are incompatible."), _("Please, choose some others."));
    return FALSE;
  }
  return TRUE;
}

/*
 * Check if container can contain multiple streams
 */
#ifdef OGMRIP_DEVEL_MODE
static gboolean
ogmrip_main_check_streams (OGMRipData *data, gint container, GList *astreams, GList *sstreams)
{
  gint nstreams;

  if (container == OGMRIP_CONTAINER_AVI)
  {
    nstreams = g_list_length (astreams);
    if (nstreams > 1)
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
          "%s", _("The AVI container does not support multiple audio streams"));
      return FALSE;
    }

    nstreams = g_list_length (sstreams);
    if (nstreams > 0)
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
          "%s", _("The AVI container does not support subtitle streams"));
      return FALSE;
    }
  }
  return TRUE;
}
#else /* OGMRIP_DEVEL_MODE */
/*
 * Checks if container can contain subtitles
 */
#ifdef HAVE_SRT_SUPPORT
static gboolean
ogmrip_main_check_subp (OGMRipData *data, OGMDvdSubpStream **sstream, gint container)
{
  if (sstream)
  {
    if (container == OGMRIP_CONTAINER_AVI)
    {
      gint response;

      response = ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_QUESTION, 
          "%s\n%s", _("The container does not support subtitles."),
          _("Do you want to continue without extracting the subtitles?"));
      if (response != GTK_RESPONSE_YES)
        return FALSE;

      g_object_unref (*sstream);
      *sstream = NULL;
    }
  }
  return TRUE;
}
#endif /* HAVE_SRT_SUPPORT */
#endif /* OGMRIP_DEVEL_MODE */

static gboolean
ogmrip_main_check_path (OGMRipData *data, const gchar *path)
{
  GError *error = NULL;

  if (!g_file_test (path, G_FILE_TEST_IS_DIR))
  {
    gint response;

    if (g_file_test (path, G_FILE_TEST_EXISTS))
    {
      response = ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_QUESTION,
          _("'%s' exists, but is not a directory.\nDo you want to remove it?"), path);
      if (response == GTK_RESPONSE_NO)
        return FALSE;
      unlink (path);
    }
    else
    {
      response = ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_QUESTION,
          _("The directory '%s' does not exist.\nDo you want to create it?"), path);
      if (response == GTK_RESPONSE_NO)
        return FALSE;
    }

    if (!g_dir_make (path, 0777, &error))
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, error->message);
      g_error_free (error);
      return FALSE;
    }
  }

  if (access (path, W_OK) == -1)
  {
    ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR, 
        _("The directory '%s' is not writable."), path);
    return FALSE;
  }

  return TRUE;
}

static gboolean
ogmrip_main_check_output (OGMRipData *data, const gchar *output)
{
  /*
   * TODO What if multiple targets ?
   */
  if (g_file_test (output, G_FILE_TEST_EXISTS))
  {
    gint response;

    response = ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_QUESTION,
        _("A file called '%s' exists.\nDo you want to overwrite it?"), output);

    if (response == GTK_RESPONSE_NO)
      return FALSE;

    unlink (output);
  }

  return TRUE;
}

/*
 * Checks space left on device
 */
static gboolean
ogmrip_main_check_space (OGMRipData *data, gint64 size, const gchar *output)
{
  gchar *output_mp, *tmp_mp;
  gboolean retval = TRUE;

  output_mp = g_fs_get_mount_point (output);
  tmp_mp = g_fs_get_mount_point (ogmrip_get_tmp_dir ());

  if (tmp_mp && output_mp && strcmp (tmp_mp, output_mp) == 0)
  {
    if (size * 2 > g_fs_space_left (output))
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
          _("Not enough space to store output and temporary files (%ldMB needed)."), 2 * size / 1024 / 1024);
      retval = FALSE;
    }
  }
  else
  {
    if (size > g_fs_space_left (ogmrip_get_tmp_dir ()))
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
          _("Not enough space to store the temporary files (%ldMB needed)."), size / 1024 / 1024);
      retval = FALSE;
    }

    if (size > g_fs_space_left (output))
    {
      ogmrip_message_dialog (GTK_WINDOW (data->window), GTK_MESSAGE_ERROR,
          _("Not enough space to store the output file (%ldMB needed)."), size / 1024 / 1024);
      retval = FALSE;
    }
  }

  g_free (output_mp);
  g_free (tmp_mp);

  return retval;
}

static OGMDvdVideoStream *
ogmrip_main_get_video_stream (OGMRipData *data)
{
  gint id;
  id = gtk_combo_box_get_active (GTK_COMBO_BOX (data->video_combo));
  return ogmdvd_get_video_stream (data->dvd, id);
}

#ifdef OGMRIP_DEVEL_MODE
static GList *
ogmrip_main_get_audio_streams (OGMRipData *data, OGMDvdVideoStream *vstream)
{
  OGMDvdAudioStream *astream;
  GList *streams, *child;
  GtkWidget *hbox, *combo;
  gint aid;

  streams = NULL;

  for (child = GTK_BOX (data->audio_vbox)->children; child; child = child->next)
  {
    hbox = GTK_BOX_CHILD (child->data)->widget;
    combo = GTK_BOX_CHILD (GTK_BOX (hbox)->children->data)->widget;

    aid = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
    if (aid > -1)
    {
      astream = ogmdvd_video_stream_get_audio_stream (vstream, aid);
      if (astream && !g_list_find (streams, astream))
        streams = g_list_append (streams, astream);
    }
  }

  return streams;
}

static GList *
ogmrip_main_get_subp_streams (OGMRipData *data, OGMDvdVideoStream *vstream)
{
  OGMDvdSubpStream *sstream;
  GList *streams, *child;
  GtkWidget *hbox, *combo;
  gint sid;

  streams = NULL;

  for (child = GTK_BOX (data->subp_vbox)->children; child; child = child->next)
  {
    hbox = GTK_BOX_CHILD (child->data)->widget;
    combo = GTK_BOX_CHILD (GTK_BOX (hbox)->children->data)->widget;

    sid = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
    if (sid > 0)
    {
      sstream = ogmdvd_video_stream_get_subp_stream (vstream, sid - 1);
      if (sstream && !g_list_find (streams, sstream))
        streams = g_list_append (streams, sstream);
    }
  }

  return streams;
}
#else /* OGMRIP_DEVEL_MODE */
static OGMDvdAudioStream *
ogmrip_main_get_audio_stream (OGMRipData *data, OGMDvdVideoStream *vstream)
{
  gint id;
  id = gtk_combo_box_get_active (GTK_COMBO_BOX (data->audio_combo));
  return ogmdvd_video_stream_get_audio_stream (vstream, id);
}

static OGMDvdSubpStream *
ogmrip_main_get_subp_stream (OGMRipData *data, OGMDvdVideoStream *vstream)
{
  gint id;
  id = gtk_combo_box_get_active (GTK_COMBO_BOX (data->subp_combo)) - 1;
  return id < 0 ? NULL : ogmdvd_video_stream_get_subp_stream (vstream, id);
}
#endif /* OGMRIP_DEVEL_MODE */

static void
ogmrip_main_get_chapter_range (OGMRipData *data, gint *start_chapter, gint *end_chapter)
{
  GtkTreeIter iter;
  gboolean extract, valid;

  valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (data->chapters_store), &iter);

  extract = *start_chapter = 0;
  while (valid && !extract)
  {
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, EXTRACT, &extract, -1);
    valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &iter);
    if (valid && !extract)
      (*start_chapter) ++;
  }

  *end_chapter = *start_chapter;
  if (!valid && !extract)
    *start_chapter = -1;

  while (valid && extract)
  {
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, EXTRACT, &extract, -1);
    valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &iter);
    if (valid && extract)
      (*end_chapter) ++;
  }

  if (!valid && extract)
    *end_chapter = -1;
}

static gchar *
ogmrip_main_get_path (OGMRipData *data)
{
  gchar *utf8, *path;

  utf8 = gconf_client_get_string_default (data->gconf, OGMRIP_GCONF_OUTPUT_DIR, OGMRIP_DEFAULT_OUTPUT_DIR);
  path = g_filename_from_utf8 (utf8, -1, NULL, NULL, NULL);
  g_free (utf8);

  return path;
}

/*
 * Returns the estimated rip size
 */
static gint64
ogmrip_main_get_size (OGMRipData *data, OGMDvdVideoStream *vstream, 
    gint start_chap, gint end_chap, gboolean relative, gint tsize, gint tnumber)
{
  glong chap_len, full_len;
  gdouble factor = 1.0;

  if (relative)
  {
    full_len = ogmdvd_video_stream_get_length (vstream, NULL);
    chap_len = ogmdvd_video_stream_get_chapters_length (vstream, NULL, start_chap, end_chap);
    factor = chap_len / full_len;
  }

  return ceil (tsize * tnumber * 1024 * 1024 * factor);
}

/*
 * Extracts sstream from start_chap to end_chap to output
 */
#ifdef HAVE_SRT_SUPPORT
static gboolean
ogmrip_main_extract_subp (OGMRipData *data, OGMDvdSubpStream *sstream, 
    gint start_chap, gint end_chap, GFile *output)
{
  OGMSpSpawn *spawn;

  spawn = ogmrip_srt_new (sstream, output->name);
  if (spawn)
  {
    ogmrip_codec_set_chapters (OGMRIP_CODEC (spawn), start_chap, end_chap);
    ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Subtitles Extraction</b>"));
    g_object_unref (spawn);

    if (ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED)
      return TRUE;
  }
  return FALSE;
}
#endif /* HAVE_SRT_SUPPORT */

/*
 * Converts astream to acodec from start_chap to end_chap to output
 */
static gboolean
ogmrip_main_extract_audio (OGMRipData *data, OGMDvdAudioStream *astream, 
    gint acodec, gint start_chap, gint end_chap, GFile *output)
{
  OGMSpSpawn *spawn = NULL;

  switch (acodec)
  {
    case OGMRIP_AUDIO_VORBIS:
      spawn = ogmrip_vorbis_new (astream, output->name);
      break;
    case OGMRIP_AUDIO_MP3:
      spawn = ogmrip_mp3_new (astream, output->name);
      break;
    case OGMRIP_AUDIO_WAV:
      spawn = ogmrip_wav_new (astream, output->name);
      break;
    case OGMRIP_AUDIO_COPY:
      spawn = ogmrip_audio_copy_new (astream, output->name);
      break;
    default:
      break;
  }

  if (spawn)
  {
    ogmrip_audio_set_quality (OGMRIP_AUDIO (spawn),
        gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_QUALITY, OGMRIP_DEFAULT_QUALITY));
    ogmrip_audio_set_normalize (OGMRIP_AUDIO (spawn),
        gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_NORMALIZE, OGMRIP_DEFAULT_NORMALIZE));
    ogmrip_codec_set_chapters (OGMRIP_CODEC (spawn), start_chap, end_chap);

    ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Audio Encoding</b>"));
    g_object_unref (spawn);

    if (ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED)
      return TRUE;
  }
  
  return FALSE;
}

/*
 * Converts vstream to vcodec from start_chap to end_chap to output
 * Bitrate is computed with nvsize, size and relative
 */
static gboolean
ogmrip_main_extract_video (OGMRipData *data, OGMDvdVideoStream *vstream,
    gint vcodec, gint start_chap, gint end_chap, gboolean relative, 
    glong nvsize, glong size, GFile *output)
{
  OGMSpSpawn *spawn = NULL;
  guint x, y, width, height, bitrate;
  gboolean retval = FALSE;

  switch (vcodec)
  {
    case OGMRIP_VIDEO_XVID:
      spawn = ogmrip_xvid_new (vstream, output->name);
      break;
    case OGMRIP_VIDEO_LAVC:
      spawn = ogmrip_lavc_new (vstream, output->name);
      break;
#ifdef HAVE_X264_SUPPORT
    case OGMRIP_VIDEO_X264:
      spawn = ogmrip_x264_new (vstream, output->name);
      break;
#endif /* HAVE_X264_SUPPORT */
#ifdef HAVE_THEORA_SUPPORT
    case OGMRIP_VIDEO_THEORA:
      break;
#endif /* HAVE_THEORA_SUPPORT */
    default:
      break;
  }

  if (spawn)
  {
    ogmrip_video_set_trellis (OGMRIP_VIDEO (spawn),
        gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_TRELLIS, OGMRIP_DEFAULT_TRELLIS));
    ogmrip_video_set_turbo (OGMRIP_VIDEO (spawn),
        gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_TURBO, OGMRIP_DEFAULT_TURBO));
    ogmrip_video_set_qpel (OGMRIP_VIDEO (spawn),
        gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_QPEL, OGMRIP_DEFAULT_QPEL));
    ogmrip_video_set_scaler (OGMRIP_VIDEO (spawn),
        gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_SCALER, OGMRIP_DEFAULT_SCALER));
    ogmrip_codec_set_chapters (OGMRIP_CODEC (spawn), start_chap, end_chap);

    ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Calculating Bitrate, Cropping, Scaling</b>"));
    g_object_unref (spawn);

    if (ogmrip_options_get_bitrate (data->options_dialog, &bitrate) == OGMRIP_OPTIONS_AUTOMATIC)
      ogmrip_video_autobitrate (OGMRIP_VIDEO (spawn), relative, nvsize, size);
    else
      ogmrip_video_set_bitrate (OGMRIP_VIDEO (spawn), bitrate);

    if (ogmrip_options_get_crop (data->options_dialog, &x, &y, &width, &height) == OGMRIP_OPTIONS_AUTOMATIC)
      ogmrip_video_autocrop (OGMRIP_VIDEO (spawn), 0);
    else
      ogmrip_video_set_crop_size (OGMRIP_VIDEO (spawn), x, y, width, height);

    if (ogmrip_options_get_scale (data->options_dialog, &width, &height) == OGMRIP_OPTIONS_AUTOMATIC)
      ogmrip_video_autoscale (OGMRIP_VIDEO (spawn));
    else
      ogmrip_video_set_scale_size (OGMRIP_VIDEO (spawn), width, height);

    if (gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_PASSES, OGMRIP_DEFAULT_PASSES) == 0)
    {
      /* Pass 0 */
      ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Video Encoding</b>"));
      retval = ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED;
    }
    else
    {
      GFile *lfile;

#if MPLAYER_PRE < 7
      /*
       * Workaround against xvid pass log file
       * Should disappear someday
       */
      if (vcodec == OGMRIP_VIDEO_XVID)
      {
        gchar *name;

        name = g_build_filename (g_get_tmp_dir (), "xvid-twopass.stats", NULL);
        lfile = g_file_open (name, NULL);
        g_free (name);
      }
      else
#endif /* MPLAYER_PRE */
        lfile = g_file_open_temp ("log.XXXXXX", NULL);

      if (!lfile)
        return FALSE;

      ogmrip_video_set_log (OGMRIP_VIDEO (spawn), lfile->name);

      /* Pass 1 */
      ogmrip_video_set_pass (OGMRIP_VIDEO (spawn), 1);
      ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Video Pass 1 Encoding</b>"));
      if (ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED)
      {
        /* Pass 2 */
        ogmrip_video_set_pass (OGMRIP_VIDEO (spawn), 2);
        ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Video Pass 2 Encoding</b>"));
        retval = ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED;
      }

      g_file_close (&lfile, TRUE);
    }
  }

  return retval;
}

/*
 * Extracts chapters of vstream from start_chap to end_chap to output
 */
static gboolean
ogmrip_main_extract_chapters (OGMRipData *data, OGMDvdVideoStream *vstream, gint start_chap, gint end_chap, GFile *output)
{
  GtkTreeIter iter;

  if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (data->chapters_store), &iter, NULL, start_chap))
  {
    GIOChannel *channel;
    glong frames, length;
    gchar *str, *title;
    gfloat rate;
    guint i;

    channel = g_io_channel_unix_new (output->fd);
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, 
        FRAMES, &frames, TITLE, &title, -1);

    str = g_strdup_printf ("CHAPTER01NAME=%s\n", title);
    g_io_channel_write_chars (channel, "CHAPTER01=00:00:00.000\n", -1, NULL, NULL);
    g_io_channel_write_chars (channel, str, -1, NULL, NULL);
    g_free (title);
    g_free (str);

    rate = ogmdvd_video_stream_get_rate (vstream);
    if (end_chap == -1)
      end_chap = ogmdvd_video_stream_get_num_chapters (vstream);

    for (i = start_chap, length = 0; i < end_chap; i++)
    {
      if (gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &iter))
      {
        length += (glong) (frames * 1000 / rate);
        str = g_strdup_printf ("CHAPTER%02d=%02lu:%02lu:%02lu.%03lu\n", i - start_chap + 2, 
            length / (60 * 60 * 1000), length / (60 * 1000) % 60, length / 1000 % 60, length % 1000);
        g_io_channel_write_chars (channel, str, -1, NULL, NULL);
        g_free (str);

        gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, 
            FRAMES, &frames, TITLE, &title, -1);

        str = g_strdup_printf ("CHAPTER%02dNAME=%s\n", i - start_chap + 2, title);
        g_io_channel_write_chars (channel, str, -1, NULL, NULL);
        g_free (title);
        g_free (str);
      }
    }

    g_io_channel_shutdown (channel, TRUE, NULL);
    g_io_channel_unref (channel);

    return TRUE;
  }

  return FALSE;
}

/*
 * Merges vfile, afile, sfile and cfile in container to output
 */
#ifdef OGMRIP_DEVEL_MODE
static gboolean
ogmrip_main_merge (OGMRipData *data, gint container, GFile *vfile, GList *audio_files,
    GList *audio_streams, GList *subp_files, GList *subp_streams, GFile *cfile, GFile *output)
{
  OGMSpSpawn *spawn;
  GList *stream, *file;
  gboolean retval;

#if MPLAYER_PRE < 7
  gchar *cwd;
#endif /* MPLAYER_PRE */

  switch (container)
  {
    case OGMRIP_CONTAINER_AVI:
      spawn = ogmrip_avi_new (output->name);
      break;
#ifdef HAVE_MKV_SUPPORT
    case OGMRIP_CONTAINER_MKV:
      spawn = ogmrip_matroska_new (output->name);
      if (gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TNUMBER, OGMRIP_DEFAULT_TNUMBER) > 1)
        ogmrip_matroska_set_tsize (OGMRIP_MATROSKA (spawn), 
            gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TSIZE, OGMRIP_DEFAULT_TSIZE));
      break;
#endif /* HAVE_MKV_SUPPORT */
    default:
      spawn = ogmrip_merge_new (output->name);
      break;
  }

  ogmrip_mux_set_video (OGMRIP_MUX (spawn), vfile->name);
  ogmrip_mux_set_title (OGMRIP_MUX (spawn), 
      gtk_entry_get_text (GTK_ENTRY (data->title_entry)));

  for (stream = audio_streams, file = audio_files; stream && file; stream = stream->next, file = file->next)
    ogmrip_mux_add_audio (OGMRIP_MUX (spawn), G_FILE (file->data)->name, 
        ogmdvd_audio_stream_get_language ((OGMDvdAudioStream *) stream->data));

  for (stream = subp_streams, file = subp_files; stream && file; stream = stream->next, file = file->next)
    ogmrip_mux_add_subp (OGMRIP_MUX (spawn), G_FILE (file->data)->name, 
        ogmdvd_subp_stream_get_language ((OGMDvdSubpStream *) stream->data));

  if (cfile)
    ogmrip_mux_add_chapters (OGMRIP_MUX (spawn), cfile->name, -1);

  ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Merging Audio and Video Streams</b>"));
  g_object_unref (spawn);

#if MPLAYER_PRE < 7
  cwd = g_get_current_dir ();
  chdir (ogmrip_get_tmp_dir ());
#endif /* MPLAYER_PRE */

  retval = ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED;

#if MPLAYER_PRE < 7
  chdir (cwd);
  g_free (cwd);
#endif /* MPLAYER_PRE */

  return retval;
}
#else /* OGMRIP_DEVEL_MODE */
static gboolean
ogmrip_main_merge (OGMRipData *data, gint container, GFile *vfile, GFile *afile, 
    gint alang, GFile *sfile, gint slang, GFile *cfile, GFile *output)
{
  OGMSpSpawn *spawn;
  gboolean retval;

#if MPLAYER_PRE < 7
  gchar *cwd;
#endif /* MPLAYER_PRE */

  switch (container)
  {
    case OGMRIP_CONTAINER_AVI:
      spawn = ogmrip_avi_new (output->name);
      break;
#ifdef HAVE_MKV_SUPPORT
    case OGMRIP_CONTAINER_MKV:
      spawn = ogmrip_matroska_new (output->name);
      if (gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TNUMBER, OGMRIP_DEFAULT_TNUMBER) > 1)
        ogmrip_matroska_set_tsize (OGMRIP_MATROSKA (spawn), 
            gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TSIZE, OGMRIP_DEFAULT_TSIZE));
      break;
#endif /* HAVE_MKV_SUPPORT */
    default:
      spawn = ogmrip_merge_new (output->name);
      break;
  }

  ogmrip_mux_set_video (OGMRIP_MUX (spawn), vfile->name);
  ogmrip_mux_set_title (OGMRIP_MUX (spawn), 
      gtk_entry_get_text (GTK_ENTRY (data->title_entry)));

  if (afile)
    ogmrip_mux_add_audio (OGMRIP_MUX (spawn), afile->name, alang);
  if (sfile)
    ogmrip_mux_add_subp (OGMRIP_MUX (spawn), sfile->name, slang);
  if (cfile)
    ogmrip_mux_add_chapters (OGMRIP_MUX (spawn), cfile->name, -1);

  ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Merging Audio and Video Streams</b>"));
  g_object_unref (spawn);

#if MPLAYER_PRE < 7
  cwd = g_get_current_dir ();
  chdir (ogmrip_get_tmp_dir ());
#endif /* MPLAYER_PRE */

  retval = ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED;

#if MPLAYER_PRE < 7
  chdir (cwd);
  g_free (cwd);
#endif /* MPLAYER_PRE */

  return retval;
}
#endif /* OGMRIP_DEVEL_MODE */

/*
 * Splits mfile in tsize size files
 */
static gboolean
ogmrip_main_split (OGMRipData *data, GFile *mfile, gint tsize, const gchar *output)
{
  OGMSpSpawn *spawn;

  spawn = ogmrip_split_new (mfile->name, output, tsize);
  ogmrip_progress_set_spawn (data->progress_dialog, spawn, _("<b>Spliting Movie</b>"));
  g_object_unref (spawn);

  return ogmsp_spawn_run (spawn) == OGMSP_RESULT_COMPLETED;
}

static void
ogmrip_main_progress_dialog_construct (OGMRipData *data, guint steps)
{
  data->progress_dialog = ogmrip_progress_new (gtk_entry_get_text (GTK_ENTRY (data->title_entry)), steps);
  g_signal_connect (data->progress_dialog, "destroy", G_CALLBACK (gtk_widget_destroyed), &data->progress_dialog);
  gtk_window_set_parent (GTK_WINDOW (data->progress_dialog), GTK_WINDOW (data->window));
}

/*
 * Events
 */

static gboolean
ogmrip_main_delete_event (GtkWidget *window, GdkEvent *event, OGMRipData *data)
{
  if (data->encoding)
    return TRUE;

  return FALSE;
}

static void
ogmrip_main_destroyed (GtkWidget *window, OGMRipData *data)
{
  if (data->gconf)
    g_object_unref (data->gconf);

  if (data->dvd)
    ogmdvd_unref (data->dvd);

  if (data->pref_dialog)
    gtk_widget_destroy (data->pref_dialog);

  if (data->options_dialog)
    gtk_widget_destroy (data->options_dialog);

  g_object_unref (data->chapters_store);
  g_free (data);
}

static void
ogmrip_main_extract_activated (GObject *object, OGMRipData *data)
{
  if (!data->progress_dialog && data->dvd)
  {
    GError *error = NULL;
    OGMDvdVideoStream *vstream;
    GFile *afile, *vfile, *cfile, *sfile, *mfile;
#ifdef OGMRIP_DEVEL_MODE
    GList *audio_streams, *subp_streams, *stream;
    GList *audio_files, *subp_files;
#endif /* OGMRIP_DEVEL_MODE */
    OGMDvdAudioStream *astream;
    OGMDvdSubpStream *sstream;

    gchar *path, *output;
    gint tsize, tnumber, passes, vcodec, acodec, lang;
    gint start_chap, end_chap, container;
    gboolean relative, keep_temp;
    glong nvsize;
    gint64 size;

    data->encoding = TRUE;

    output = NULL;
    afile = vfile = cfile = sfile = mfile = NULL;
    nvsize = 0;

    vstream = ogmrip_main_get_video_stream (data);
#ifdef OGMRIP_DEVEL_MODE
    audio_files = subp_files = NULL;
    audio_streams = ogmrip_main_get_audio_streams (data, vstream);
    subp_streams = ogmrip_main_get_subp_streams (data, vstream);
#else /* OGMRIP_DEVEL_MODE */
    astream = ogmrip_main_get_audio_stream (data, vstream);
    sstream = ogmrip_main_get_subp_stream (data, vstream);
#endif /* OGMRIP_DEVEL_MODE */

    ogmrip_main_get_chapter_range (data, &start_chap, &end_chap);
    relative = GTK_WIDGET_SENSITIVE (data->relative_check) & 
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->relative_check));

    keep_temp = gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_KEEP_TEMP, OGMRIP_DEFAULT_KEEP_TEMP);

    container = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_CONTAINER, OGMRIP_DEFAULT_CONTAINER);
    vcodec = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_VIDEO, OGMRIP_DEFAULT_VIDEO);
    acodec = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_AUDIO, OGMRIP_DEFAULT_AUDIO);

    passes = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_PASSES, OGMRIP_DEFAULT_PASSES);
    tnumber = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TNUMBER, OGMRIP_DEFAULT_TNUMBER);
    tsize = gconf_client_get_int_default (data->gconf, OGMRIP_GCONF_TSIZE, OGMRIP_DEFAULT_TSIZE);

    if (gtk_dialog_run (GTK_DIALOG (data->options_dialog)) != GTK_RESPONSE_OK)
      goto extract_cleanup;

    /*
     * Check if container and codecs are compatible
     */
    if (!ogmrip_main_check_container (data, container, vcodec, acodec))
      goto extract_cleanup;

#ifdef OGMRIP_DEVEL_MODE
    /*
     * Check if container can contain multiple streams
     */
    if (!ogmrip_main_check_streams (data, container, audio_streams, subp_streams))
      goto extract_cleanup;
#endif /* OGMRIP_DEVEL_MODE */

    path = ogmrip_main_get_path (data);

    /*
     * Check output path
     */
    if (!ogmrip_main_check_path (data, path))
    {
      g_free (path);
      goto extract_cleanup;
    }

    /*
     * Build filename
     */
#ifdef OGMRIP_DEVEL_MODE
    lang = ogmdvd_audio_stream_get_language ((OGMDvdAudioStream *) audio_streams->data);
#else
    lang = ogmdvd_audio_stream_get_language (astream);
#endif
    output = ogmrip_main_build_filename (data, path, container, vcodec, acodec, lang);
    g_free (path);

    /*
     * Check output file
     */
    if (!ogmrip_main_check_output (data, output))
    {
      g_free (output);
      goto extract_cleanup;
    }

    /*
     * Check space left on device
     */
    size = ogmrip_main_get_size (data, vstream, start_chap, end_chap, relative, tsize, tnumber);

    if (!ogmrip_main_check_space (data, size, output))
      goto extract_cleanup;

#ifdef OGMRIP_DEVEL_MODE
    ogmrip_main_progress_dialog_construct (data, 
        3 + passes + (tnumber > 1 ? 1 : 0) + g_list_length (audio_streams) + g_list_length (subp_streams));
#else /* OGMRIP_DEVEL_MODE */
    ogmrip_main_progress_dialog_construct (data, 4 + passes + (tnumber > 1 ? 1 : 0) + (sstream ? 1 : 0));
#endif /* OGMRIP_DEVEL_MODE */
    gtk_widget_show (data->progress_dialog);

#ifdef HAVE_SRT_SUPPORT
    /*
     * Subtitles Extraction
     */
#ifdef OGMRIP_DEVEL_MODE
    for (stream = subp_streams; stream; stream = stream->next)
    {
      sstream = (OGMDvdSubpStream *) stream->data;

      sfile = g_file_open_temp ("sub.XXXXXX", &error);
      if (!sfile)
        goto extract_cleanup;

      if (!ogmrip_main_extract_subp (data, sstream, start_chap, end_chap, sfile))
        goto extract_cleanup;

#ifdef HAVE_ENCHANT_SUPPORT
      if (gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_SPELL_CHECK, OGMRIP_DEFAULT_SPELL_CHECK))
        ogmrip_main_spell_check (data, sfile,
            ogmdvd_get_language_iso639_1 (ogmdvd_subp_stream_get_language (sstream)));
#endif /* HAVE_ENCHANT_SUPPORT */

      subp_files = g_list_append (subp_files, sfile);

      /*
       * Non video size for auto bitrate calculation
       */
      nvsize += sfile ? g_file_get_size (sfile) : 0;
    }
#else /* OGMRIP_DEVEL_MODE */
    if (sstream)
    {
      if (!ogmrip_main_check_subp (data, &sstream, container))
        goto extract_cleanup;

      sfile = g_file_open_temp ("sub.XXXXXX", &error);
      if (!sfile)
        goto extract_cleanup;

      if (!ogmrip_main_extract_subp (data, sstream, start_chap, end_chap, sfile))
        goto extract_cleanup;

#ifdef HAVE_ENCHANT_SUPPORT
      if (gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_SPELL_CHECK, OGMRIP_DEFAULT_SPELL_CHECK))
        ogmrip_main_spell_check (data, sfile,
            ogmdvd_get_language_iso639_1 (ogmdvd_subp_stream_get_language (sstream)));
#endif /* HAVE_ENCHANT_SUPPORT */
    }
#endif /* OGMRIP_DEVEL_MODE */
#endif /* HAVE_SRT_SUPPORT */

    /*
     * Audio encoding
     */
    if (!gconf_client_get_bool_default (data->gconf, OGMRIP_GCONF_NOAUDIO, OGMRIP_DEFAULT_NOAUDIO))
    {
#ifdef OGMRIP_DEVEL_MODE
      for (stream = audio_streams; stream; stream = stream->next)
      {
        astream = (OGMDvdAudioStream *) stream->data;

        if (container == OGMRIP_CONTAINER_AVI && acodec == OGMRIP_AUDIO_COPY)
          afile = g_file_open_temp ("frameno.XXXXXX", &error);
        else
          afile = g_file_open_temp ("audio.XXXXXX", &error);

        if (!afile)
          goto extract_cleanup;

        if (!ogmrip_main_extract_audio (data, astream, acodec, start_chap, end_chap, afile))
          goto extract_cleanup;

        audio_files = g_list_append (audio_files, afile);

        /*
         * Non video size for auto bitrate calculation
         */
        nvsize += afile ? g_file_get_size (afile) : 0;
      }
#else /* OGMRIP_DEVEL_MODE */
      if (container == OGMRIP_CONTAINER_AVI && acodec == OGMRIP_AUDIO_COPY)
        afile = g_file_open_temp ("frameno.XXXXXX", &error);
      else
        afile = g_file_open_temp ("audio.XXXXXX", &error);

      if (!afile)
        goto extract_cleanup;

      if (!ogmrip_main_extract_audio (data, astream, acodec, start_chap, end_chap, afile))
        goto extract_cleanup;
#endif /* OGMRIP_DEVEL_MODE */
    }

    /*
     * Video encoding
     */
    vfile = g_file_open_temp ("video.XXXXXX", &error);
    if (!vfile)
      goto extract_cleanup;

    /*
     * Non video size for auto bitrate calculation
     */
#ifndef OGMRIP_DEVEL_MODE
    nvsize = afile ? g_file_get_size (afile) : 0;
    nvsize += sfile ? g_file_get_size (sfile) : 0;
#endif /* OGMRIP_DEVEL_MODE */

    if (!ogmrip_main_extract_video (data, vstream, vcodec, start_chap, end_chap, relative, 
          nvsize, tsize * tnumber * 1024 * 1024, vfile))
      goto extract_cleanup;

    /*
     * Chapters extraction
     */
    cfile = g_file_open_temp ("chapters.XXXXXX", &error);
    if (!cfile)
      goto extract_cleanup;

    if (!ogmrip_main_extract_chapters (data, vstream, start_chap, end_chap, cfile))
      goto extract_cleanup;

    /*
     * Merge
     */
    if (tnumber == 1)
      mfile = g_file_open (output, &error);
    else
      mfile = g_file_open_temp ("merge.XXXXXX", &error);

    if (!mfile)
      goto extract_cleanup;

#ifdef OGMRIP_DEVEL_MODE

#if MPLAYER_PRE < 7
    if (container == OGMRIP_CONTAINER_AVI && acodec == OGMRIP_AUDIO_COPY)
    {
      gchar *frameno;

      frameno = g_build_filename (ogmrip_get_tmp_dir (), "frameno.avi", NULL);
      g_file_rename (G_FILE (audio_files->data), frameno, NULL);
      g_free (frameno);
    }
#endif /* MPLAYER_PRE */
    if (!ogmrip_main_merge (data, container, vfile, audio_files, 
          audio_streams, subp_files, subp_streams, cfile, mfile))
      goto extract_cleanup;

#else /* OGMRIP_DEVEL_MODE */

#if MPLAYER_PRE < 7
    if (container == OGMRIP_CONTAINER_AVI && acodec == OGMRIP_AUDIO_COPY)
    {
      gchar *frameno;

      frameno = g_build_filename (ogmrip_get_tmp_dir (), "frameno.avi", NULL);
      g_file_rename (afile, frameno, NULL);
      g_free (frameno);
    }
#endif /* MPLAYER_PRE */
    if (!ogmrip_main_merge (data, container, vfile, 
          afile, ogmdvd_audio_stream_get_language (astream), 
          sfile, sstream ? ogmdvd_subp_stream_get_language (sstream) : -1, cfile, mfile)) 
      goto extract_cleanup;

#endif /* OGMRIP_DEVEL_MODE */

    /*
     * Remove unneeded temporary files
     */
    g_file_close (&afile, !keep_temp);
    g_file_close (&vfile, !keep_temp);
    g_file_close (&cfile, !keep_temp);
    g_file_close (&sfile, !keep_temp);

    if (container == OGMRIP_CONTAINER_OGM && tnumber > 1)
    {
      /*
       * Split
       */
      if (!ogmrip_main_split (data, mfile, tsize, output))
        goto extract_cleanup;
    }

extract_cleanup:
    if (data->progress_dialog)
      gtk_widget_destroy (data->progress_dialog);

    if (vstream)
      ogmdvd_video_stream_unref (vstream);

#ifdef OGMRIP_DEVEL_MODE
    g_list_foreach (audio_streams, 
        (GFunc) ogmdvd_audio_stream_unref, NULL);
    g_list_foreach (subp_streams, 
        (GFunc) ogmdvd_subp_stream_unref, NULL);
    g_list_foreach (audio_files, 
        (GFunc) g_file_close, GINT_TO_POINTER (!keep_temp));
    g_list_foreach (subp_files, 
        (GFunc) g_file_close, GINT_TO_POINTER (!keep_temp));
#else /* OGMRIP_DEVEL_MODE */
    if (astream)
      ogmdvd_audio_stream_unref (astream);

    if (sstream)
      ogmdvd_subp_stream_unref (sstream);

    g_file_close (&afile, !keep_temp);
    g_file_close (&vfile, !keep_temp);
#endif /* OGMRIP_DEVEL_MODE */

    if (error)
      g_error_free (error);

    if (output)
      g_free (output);

    g_file_close (&cfile, !keep_temp);
    g_file_close (&sfile, !keep_temp);
    g_file_close (&mfile, tnumber > 1 && !keep_temp);

    data->encoding = FALSE;
  }
}

static void
ogmrip_main_load_activated (GObject *object, OGMRipData *data)
{
  if (!data->encoding)
  {
    if (gtk_dialog_run (GTK_DIALOG (data->open_disk_dialog)) == GTK_RESPONSE_OK)
    {
      const gchar *device;
      
      device = ogmrip_dvd_chooser_get_device (OGMRIP_DVD_CHOOSER (data->dvd_chooser));
      if (device)
        ogmrip_main_load (data, TRUE, device);
    }
  }
}

static void
ogmrip_main_open_activated (GObject *object, OGMRipData *data)
{
  if (!data->encoding)
  {
    if (gtk_dialog_run (GTK_DIALOG (data->open_struct_dialog)) == GTK_RESPONSE_OK)
    {
      gchar *dir;

      dir = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->open_struct_dialog));
      if (dir)
        ogmrip_main_load (data, FALSE, dir);
    }
  }
}

static void
ogmrip_main_select_all_activated (GtkAction *action, OGMRipData *data)
{
  OGMDvdVideoStream *vstream;
  OGMDvdTime time_;

  GtkTreeModel *model;
  GtkTreeIter iter;
  gint vid;

  model = GTK_TREE_MODEL (data->chapters_store);
  if (gtk_tree_model_get_iter_first (model, &iter))
  {
    do
    {
      gtk_list_store_set (data->chapters_store, &iter, EXTRACT, TRUE, -1);
    }
    while (gtk_tree_model_iter_next (model, &iter));
  }

  vid = gtk_combo_box_get_active (GTK_COMBO_BOX (data->video_combo));
  vstream = ogmdvd_get_video_stream (data->dvd, vid);
  if (ogmdvd_video_stream_get_length (vstream, &time_) > 0)
  {
    gchar *str;

    str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
    gtk_label_set_text (GTK_LABEL (data->length_label), str);
    g_free (str);
  }
  ogmdvd_video_stream_unref (vstream);

  gtk_widget_set_sensitive (data->extract_button, TRUE);
  g_object_set (data->extract_action, "sensitive", TRUE, NULL);
  gtk_widget_set_sensitive (data->relative_check, FALSE);
}

static void
ogmrip_main_deselect_all_activated (GtkAction *action, OGMRipData *data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;

  model = GTK_TREE_MODEL (data->chapters_store);
  if (gtk_tree_model_get_iter_first (model, &iter))
  {
    do
    {
      gtk_list_store_set (data->chapters_store, &iter, EXTRACT, FALSE, -1);
    }
    while (gtk_tree_model_iter_next (model, &iter));
  }

  gtk_label_set_text (GTK_LABEL (data->length_label), "");
  gtk_widget_set_sensitive (data->extract_button, FALSE);
  g_object_set (data->extract_action, "sensitive", FALSE, NULL);
  gtk_widget_set_sensitive (data->relative_check, FALSE);
}

static void
ogmrip_main_pref_activated (GtkAction *action, OGMRipData *data)
{
  gtk_widget_show (data->pref_dialog);
}

static void
ogmrip_main_about_activated (GtkAction *action, OGMRipData *data)
{
  static GtkWidget *dialog = NULL;
  const gchar *authors[] = 
  { 
    "Olivier Rolland <billl@users.sf.net>", 
    NULL 
  };
  gchar *translator_credits = _("translator-credits");

  if (!dialog)
  {
    GdkPixbuf *icon;

    icon = gdk_pixbuf_new_from_file (OGMRIP_DATA_DIR "/" OGMRIP_ICON_FILE, NULL);
    dialog = gnome_about_new (PACKAGE_NAME, PACKAGE_VERSION, "(c) 2004 Olivier Rolland", 
        _("A DVD Encoder for GNOME"), authors, NULL, 
        strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL, icon);
    g_object_unref (icon);

    gtk_window_set_parent (GTK_WINDOW (dialog), GTK_WINDOW (data->window));
    g_signal_connect (dialog, "destroy", G_CALLBACK (gtk_widget_destroyed), &dialog);
    gtk_widget_show (dialog);
  }
}

static void
ogmrip_main_video_combo_changed (GtkComboBox *combo, OGMRipData *data)
{
  GtkWidget *audio_combo;
  GtkWidget *subp_combo;

  OGMDvdVideoStream *vstream;
  OGMDvdTime time_;
  gchar *str;
  gint vid;

#ifndef OGMRIP_DEVEL_MODE
  GtkTreeModel *model;
#endif /* OGMRIP_DEVEL_MODE */

  gtk_list_store_clear (data->chapters_store);

#ifdef OGMRIP_DEVEL_MODE
  gtk_container_foreach (GTK_CONTAINER (data->audio_vbox), 
      (GtkCallback) ogmrip_main_remove_stream, NULL);
  gtk_container_foreach (GTK_CONTAINER (data->subp_vbox), 
      (GtkCallback) ogmrip_main_remove_stream, NULL);

  audio_combo = gtk_combo_box_new_text ();
  ogmrip_main_add_stream (data, audio_combo, data->audio_vbox);
  gtk_widget_show (audio_combo);

  subp_combo = gtk_combo_box_new_text ();
  ogmrip_main_add_stream (data, subp_combo, data->subp_vbox);
  gtk_widget_show (subp_combo);
#else /* OGMRIP_DEVEL_MODE */
  audio_combo = data->audio_combo;
  model = gtk_combo_box_get_model (GTK_COMBO_BOX (audio_combo));
  gtk_list_store_clear (GTK_LIST_STORE (model));

  subp_combo = data->subp_combo;
  model = gtk_combo_box_get_model (GTK_COMBO_BOX (subp_combo));
  gtk_list_store_clear (GTK_LIST_STORE (model));
#endif /* OGMRIP_DEVEL_MODE */

  gtk_label_set_text (GTK_LABEL (data->length_label), "");

  if (data->dvd)
  {
    gint pref;

    vid = gtk_combo_box_get_active (combo);
    vstream = ogmdvd_get_video_stream (data->dvd, vid);

    if (ogmdvd_video_stream_get_length (vstream, &time_) > 0)
    {
      str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
      gtk_label_set_text (GTK_LABEL (data->length_label), str);
      g_free (str);
    }

    pref = gconf_client_get_int_default (data->gconf, 
        OGMRIP_GCONF_PREF_AUDIO, OGMRIP_DEFAULT_PREF_AUDIO);
    ogmrip_main_fill_audio_combo (audio_combo, vstream, pref);

    pref = gconf_client_get_int_default (data->gconf, 
        OGMRIP_GCONF_PREF_SUBP, OGMRIP_DEFAULT_PREF_SUBP);
    ogmrip_main_fill_subp_combo (subp_combo, vstream, pref);

    ogmrip_main_fill_chapters_list (data->chapters_store, vstream);

    ogmrip_options_set_video_stream (data->options_dialog, vstream);

    ogmdvd_video_stream_unref (vstream);

    gtk_widget_set_sensitive (data->relative_check, FALSE);
  }
}

#ifdef OGMRIP_DEVEL_MODE
static void
ogmrip_main_add_stream_clicked (GtkWidget *button, OGMRipData *data)
{
  GtkWidget *vbox, *combo;
  OGMDvdVideoStream *vstream;
  gint vid;

  vbox = button->parent->parent->parent;
  gtk_widget_hide (button);

  button = GTK_BOX_CHILD (GTK_BOX (button->parent)->children->next->data)->widget;
  gtk_widget_show (button);

  combo = gtk_combo_box_new_text ();
  ogmrip_main_add_stream (data, combo, vbox);
  gtk_widget_show (combo);

  vid = gtk_combo_box_get_active (GTK_COMBO_BOX (data->video_combo));
  vstream = ogmdvd_get_video_stream (data->dvd, vid);

  if (vbox == data->audio_vbox)
    ogmrip_main_fill_audio_combo (combo, vstream, 0);
  else
    ogmrip_main_fill_subp_combo (combo, vstream, 0);
}

static void
ogmrip_main_remove_stream_clicked (GtkWidget *button, OGMRipData *data)
{
  ogmrip_main_remove_stream (button->parent->parent);
}
#endif /* OGMRIP_DEVEL_MODE */

static void
ogmrip_main_extract_cell_toggled (GtkCellRendererToggle *cell, gchar *path, OGMRipData *data)
{
  GtkTreeIter iter, next_iter, prev_iter;
  gboolean extract, next_extract, prev_extract;
  gint start_chap, end_chap;

  OGMDvdVideoStream *vstream;
  OGMDvdTime time_;
  gchar *str;
  gint vid;

  next_extract = prev_extract = FALSE;

  gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (data->chapters_store), &iter, path);

  next_iter = iter;
  if (gtk_tree_model_iter_next (GTK_TREE_MODEL (data->chapters_store), &next_iter))
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &next_iter, EXTRACT, &next_extract, -1);

  prev_iter = iter;
  if (gtk_tree_model_iter_prev (GTK_TREE_MODEL (data->chapters_store), &prev_iter))
    gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &prev_iter, EXTRACT, &prev_extract, -1);

  if ((prev_extract && next_extract) || (!prev_extract && !next_extract))
    ogmrip_main_deselect_all_activated (NULL, data);

  gtk_tree_model_get (GTK_TREE_MODEL (data->chapters_store), &iter, EXTRACT, &extract, -1);
  gtk_list_store_set (data->chapters_store, &iter, EXTRACT, extract ? FALSE : TRUE, -1);

  ogmrip_main_get_chapter_range (data, &start_chap, &end_chap);

  vid = gtk_combo_box_get_active (GTK_COMBO_BOX (data->video_combo));
  vstream = ogmdvd_get_video_stream (data->dvd, vid);
  if (ogmdvd_video_stream_get_chapters_length (vstream, &time_, start_chap, end_chap) > 0)
  {
    str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
    gtk_label_set_text (GTK_LABEL (data->length_label), str);
    g_free (str);
  }
  ogmdvd_video_stream_unref (vstream);

  gtk_widget_set_sensitive (data->extract_button, start_chap != -1);
  g_object_set (data->extract_action, "sensitive", start_chap != -1, NULL);

  gtk_widget_set_sensitive (data->relative_check, start_chap > 0 || end_chap != -1);
}

static void
ogmrip_main_title_cell_edited (GtkCellRendererText *cell, gchar *path, gchar *text, OGMRipData *data)
{
  GtkTreeIter iter;

  gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (data->chapters_store), &iter, path);
  gtk_list_store_set (data->chapters_store, &iter, TITLE, text, -1);
}

static gboolean
ogmrip_main_item_enter (GtkWidget *item, GdkEvent *event, OGMRipData *data)
{
  gchar *hint;

  hint = g_object_get_data (G_OBJECT (item), "__menu_hint__");
  if (hint)
  {
    gnome_appbar_push (GNOME_APPBAR (data->appbar), hint);
  }

  return FALSE;
}

static gboolean
ogmrip_main_item_leave (GtkWidget *item, GdkEvent *event, OGMRipData *data)
{
  gnome_appbar_pop (GNOME_APPBAR (data->appbar));

  return FALSE;
}

static GtkAction *
ogmrip_main_action_connect (OGMRipData *data, const gchar *path,
        GCallback callback, gpointer user_data, gboolean sensitive, gboolean swapped)
{
  GtkAction *action;

  action = gtk_ui_manager_get_action (data->ui_manager, path);
  if (action)
  {
    gchar *hint;

    g_object_set (action, "sensitive", sensitive, NULL);

    if (callback)
    {
      if (swapped)
        g_signal_connect_swapped (action, "activate", callback, user_data);
      else
        g_signal_connect (action, "activate", callback, user_data);
    }

    g_object_get (action, "tooltip", &hint, NULL);
    if (hint)
    {
      GtkWidget *widget;

      widget = gtk_ui_manager_get_widget (data->ui_manager, path);
      g_object_set_data_full (G_OBJECT (widget), "__menu_hint__", hint, (GDestroyNotify) g_free);

      g_signal_connect (widget, "enter-notify-event", G_CALLBACK (ogmrip_main_item_enter), data);
      g_signal_connect (widget, "leave-notify-event", G_CALLBACK (ogmrip_main_item_leave), data);
    }
  }
  return action;
}

static OGMRipData *
ogmrip_main_new (void)
{
  OGMRipData *data;
  GtkWidget *widget;
  GtkAction *action;
  GladeXML *xml;

  GtkAccelGroup *accel_group;
  GtkTreeViewColumn *column;
  GtkCellRenderer *renderer;

  xml = glade_xml_new (OGMRIP_DATA_DIR "/" OGMRIP_GLADE_FILE, NULL, NULL);
  if (!xml)
  {
    g_warning ("Could not find " OGMRIP_GLADE_FILE);
    return NULL;
  }

  data = g_new0 (OGMRipData, 1);

  data->gconf = gconf_client_get_default ();
  gconf_client_add_dir (data->gconf, OGMRIP_GCONF_ROOT, 
      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

  data->window = glade_xml_get_widget (xml, "main-window");
  gtk_window_set_default_size (GTK_WINDOW (data->window), 350, 500);
  gtk_window_set_icon_from_file (GTK_WINDOW (data->window), OGMRIP_DATA_DIR "/" OGMRIP_ICON_FILE, NULL);

  g_signal_connect (data->window, "delete-event", G_CALLBACK (ogmrip_main_delete_event), data);
  g_signal_connect (data->window, "destroy", G_CALLBACK (ogmrip_main_destroyed), data);
  g_signal_connect (data->window, "destroy", G_CALLBACK (gtk_main_quit), NULL);

  data->action_group = gtk_action_group_new ("MenuActions");
  gtk_action_group_set_translation_domain (data->action_group, GETTEXT_PACKAGE);
  gtk_action_group_add_actions (data->action_group, action_entries, 
      G_N_ELEMENTS (action_entries), data->window);

  data->ui_manager = gtk_ui_manager_new ();
  gtk_ui_manager_insert_action_group (data->ui_manager, data->action_group, 0);
  gtk_ui_manager_add_ui_from_string (data->ui_manager, ui_description, -1, NULL);

  accel_group = gtk_ui_manager_get_accel_group (data->ui_manager);
  gtk_window_add_accel_group (GTK_WINDOW (data->window), accel_group);

  widget = gtk_ui_manager_get_widget (data->ui_manager, "/Menubar");
  gnome_app_set_menus (GNOME_APP (data->window), GTK_MENU_BAR (widget));

  data->extract_action = ogmrip_main_action_connect (data, "/Menubar/FileMenu/Extract", 
      G_CALLBACK (ogmrip_main_extract_activated), data, FALSE, FALSE);

  data->extract_button = glade_xml_get_widget (xml, "extract-button");
  g_signal_connect (data->extract_button, "clicked", 
      G_CALLBACK (ogmrip_main_extract_activated), data);

  action = ogmrip_main_action_connect (data, "/Menubar/FileMenu/Load", 
      G_CALLBACK (ogmrip_main_load_activated), data, TRUE, FALSE);

  widget = glade_xml_get_widget (xml, "load-button");
  g_signal_connect (widget, "clicked", 
      G_CALLBACK (ogmrip_main_load_activated), data);

  action = ogmrip_main_action_connect (data, "/Menubar/FileMenu/Open", 
      G_CALLBACK (ogmrip_main_open_activated), data, TRUE, FALSE);

  widget = glade_xml_get_widget (xml, "open-button");
  g_signal_connect (widget, "clicked", 
      G_CALLBACK (ogmrip_main_open_activated), data);

  ogmrip_main_action_connect (data, "/Menubar/FileMenu/Quit", 
      G_CALLBACK (gtk_widget_destroy), data->window, TRUE, TRUE);

  ogmrip_main_action_connect (data, "/Menubar/EditMenu/Preferences", 
      G_CALLBACK (ogmrip_main_pref_activated), data, TRUE, FALSE);

  ogmrip_main_action_connect (data, "/Menubar/EditMenu/SelectAll", 
      G_CALLBACK (ogmrip_main_select_all_activated), data, TRUE, FALSE);

  ogmrip_main_action_connect (data, "/Menubar/EditMenu/DeselectAll", 
      G_CALLBACK (ogmrip_main_deselect_all_activated), data, TRUE, FALSE);

  ogmrip_main_action_connect (data, "/Menubar/HelpMenu/About", 
      G_CALLBACK (ogmrip_main_about_activated), data, TRUE, FALSE);

  data->title_entry = glade_xml_get_widget (xml, "title-entry");

  data->video_combo = glade_xml_get_widget (xml, "video-combo");
  g_signal_connect (data->video_combo, "changed", 
      G_CALLBACK (ogmrip_main_video_combo_changed), data);

#ifdef OGMRIP_DEVEL_MODE
  data->subp_vbox = glade_xml_get_widget (xml, "subp-vbox");
  data->audio_vbox = glade_xml_get_widget (xml, "audio-vbox");

  gtk_box_set_spacing (GTK_BOX (glade_xml_get_widget (xml, "subp-hbox")), 6);
  gtk_box_set_spacing (GTK_BOX (glade_xml_get_widget (xml, "audio-hbox")), 6);

  gtk_widget_show (glade_xml_get_widget (xml, "audio-add-button"));
  gtk_widget_show (glade_xml_get_widget (xml, "subp-add-button"));
#else /* OGMRIP_DEVEL_MODE */
  data->subp_combo = glade_xml_get_widget (xml, "subp-combo");
  data->audio_combo = glade_xml_get_widget (xml, "audio-combo");

  gtk_widget_set_sensitive (glade_xml_get_widget (xml, "audio-hbox"), TRUE);
  gtk_widget_set_sensitive (glade_xml_get_widget (xml, "subp-hbox"), TRUE);
#endif /* OGMRIP_DEVEL_MODE */
  data->length_label = glade_xml_get_widget (xml, "length-label");
  data->relative_check = glade_xml_get_widget (xml, "relative-check");

  widget = glade_xml_get_widget (xml, "treeview");

  data->chapters_store = gtk_list_store_new (COL_LAST, 
      G_TYPE_BOOLEAN, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_POINTER);
  gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (data->chapters_store));

  renderer = gtk_cell_renderer_toggle_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Extract?"), renderer, "active", EXTRACT, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
  g_object_set (renderer, "activatable", TRUE, NULL);

  g_signal_connect (renderer, "toggled", G_CALLBACK (ogmrip_main_extract_cell_toggled), data);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Chapter"), renderer, "text", CHAPTER, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Duration"), renderer, "text", LENGTH, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Title"), renderer, "text", TITLE, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
  gtk_tree_view_column_set_resizable (column, TRUE);
  g_object_set (renderer, "editable", TRUE, NULL);

  g_signal_connect (renderer, "edited", G_CALLBACK (ogmrip_main_title_cell_edited), data);
  
  data->appbar = glade_xml_get_widget (xml, "appbar");

  data->open_disk_dialog = glade_xml_get_widget (xml, "open-disk-dialog");
  gtk_window_set_icon_from_stock (GTK_WINDOW (data->open_disk_dialog), GTK_STOCK_REFRESH);
  gtk_window_set_parent (GTK_WINDOW (data->open_disk_dialog), GTK_WINDOW (data->window));
  g_signal_connect (data->open_disk_dialog, "response", G_CALLBACK (gtk_widget_hide), NULL);

  data->dvd_chooser = glade_xml_get_widget (xml, "dvd-chooser");
  gtk_widget_show (data->dvd_chooser);

  data->open_struct_dialog = gtk_file_chooser_dialog_new (_("Open DVD Structure"), 
      GTK_WINDOW (data->window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, 
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
  gtk_window_set_icon_from_stock (GTK_WINDOW (data->open_struct_dialog), GTK_STOCK_OPEN);
  gtk_window_set_parent (GTK_WINDOW (data->open_struct_dialog), GTK_WINDOW (data->window));
  g_signal_connect (data->open_struct_dialog, "response", G_CALLBACK (gtk_widget_hide), NULL);

  g_object_unref (xml);

  return data;
}

int
main (int argc, char *argv[])
{
  OGMRipData *data;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif /* ENABLE_NLS */

  gnome_program_init (PACKAGE_NAME, PACKAGE_VERSION, 
      LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_NONE);

  data = ogmrip_main_new ();

  data->pref_dialog = ogmrip_pref_new ();
  gtk_window_set_parent (GTK_WINDOW (data->pref_dialog), GTK_WINDOW (data->window));

  data->options_dialog = ogmrip_options_new ();
  gtk_window_set_parent (GTK_WINDOW (data->options_dialog), GTK_WINDOW (data->window));

  gtk_widget_show (data->window);

  gtk_main ();

  return 0;
}

