/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include "ogmdvd.h"

#include "ogmrip-pref.h"
#include "ogmrip-gconf.h"
#include "ogmrip-helper.h"

#include <gnome.h>
#include <glade/glade.h>
#include <string.h>

#define OGMRIP_GLADE_FILE "ogmrip/ogmrip-pref.glade"

extern gchar *ogmdvd_languages[][3];
extern guint ogmdvd_nlanguages;

static void
ogmrip_pref_set_active_lang (GtkComboBox *combo, gint code)
{
  gint i = 0;

  if (code)
  {
    const gchar *lang;

    lang = ogmdvd_get_language_iso639_1 (code);
    for (i = 1; i < ogmdvd_nlanguages; i++)
      if (strcmp (ogmdvd_languages[i + 1][OGMDVD_LANGUAGE_ISO639_1], lang) == 0)
        break;

    if (i == ogmdvd_nlanguages)
      i = 0;
  }

  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
}

static gint
ogmrip_pref_get_active_lang (GtkComboBox *combo)
{
  const gchar *lang;
  gint i;

  i = gtk_combo_box_get_active (combo);
  if (i > 0 && i + 1 < ogmdvd_nlanguages)
  {
    lang = ogmdvd_languages[i + 1][OGMDVD_LANGUAGE_ISO639_1];
    return (lang[0] << 8) | lang[1];
  }

  return 0;
}

static void
ogmrip_pref_output_browse_clicked (GtkWidget *button, GConfClient *gconf)
{
  GtkWidget *dialog;
  gchar *path;

  dialog = gtk_file_chooser_dialog_new ("Select a directory", 
      NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, 
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, 
      NULL);

  path = gconf_client_get_string (gconf, OGMRIP_GCONF_OUTPUT_DIR, NULL);
  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), path);
  g_free (path);

  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
  {
    path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    gconf_client_set_string (gconf, OGMRIP_GCONF_OUTPUT_DIR, path, NULL);
    g_free (path);
  }

  gtk_widget_destroy (dialog);
}

static void
ogmrip_pref_tmp_browse_clicked (GtkWidget *button, GConfClient *gconf)
{
  GtkWidget *dialog;
  gchar *path;

  dialog = gtk_file_chooser_dialog_new ("Select a directory", 
      NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, 
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, 
      NULL);

  path = gconf_client_get_string (gconf, OGMRIP_GCONF_TMP_DIR, NULL);
  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), path);
  g_free (path);

  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
  {
    path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    gconf_client_set_string (gconf, OGMRIP_GCONF_TMP_DIR, path, NULL);
    ogmrip_set_tmp_dir (path);
    g_free (path);
  }

  gtk_widget_destroy (dialog);
}

static void
ogmrip_pref_output_dir_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *widget)
{
  /*
   * Sanity checks
   */
  gtk_entry_set_text (GTK_ENTRY (widget),
      gconf_value_get_string (entry->value));
}

static void
ogmrip_pref_tmp_dir_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *widget)
{
  /*
   * Sanity checks
   */
  gtk_entry_set_text (GTK_ENTRY (widget),
      gconf_value_get_string (entry->value));
}

static void
ogmrip_pref_filename_changed (GtkWidget *combo, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_FILENAME, 
      gtk_combo_box_get_active (GTK_COMBO_BOX (combo)), NULL);
}

static void
ogmrip_pref_filename_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *combo)
{
  gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
      CLAMP (gconf_value_get_int (entry->value), 0, 3));
}

static void
ogmrip_pref_container_changed (GtkWidget *combo, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_CONTAINER, 
      gtk_combo_box_get_active (GTK_COMBO_BOX (combo)), NULL);
}

static void
ogmrip_pref_container_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *combo)
{
  gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
      CLAMP (gconf_value_get_int (entry->value), 0, 2));
}

static void
ogmrip_pref_target_number_changed (GtkWidget *spin, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_TNUMBER, 
      gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)), NULL);
}

static void
ogmrip_pref_target_number_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *spin)
{
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin),
      MAX (1, gconf_value_get_int (entry->value)));
}

static void
ogmrip_pref_target_size_changed (GtkWidget *spin, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_TSIZE, 
      gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)), NULL);
}

static void
ogmrip_pref_target_size_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *spin)
{
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin),
      MAX (1, gconf_value_get_int (entry->value)));
}

static void
ogmrip_pref_video_format_toggled (GtkWidget *radio, GConfClient *gconf)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)))
    gconf_client_set_int (gconf, OGMRIP_GCONF_VIDEO,
        gtk_radio_button_get_active (GTK_RADIO_BUTTON (radio)), NULL);
}

static void
ogmrip_pref_video_format_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *radio)
{
  gtk_radio_button_set_active (GTK_RADIO_BUTTON (radio),
      CLAMP (gconf_value_get_int (entry->value), 0, 2));
}

static void
ogmrip_pref_passes_changed (GtkWidget *combo, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_PASSES, 
      gtk_combo_box_get_active (GTK_COMBO_BOX (combo)), NULL);
}

static void
ogmrip_pref_passes_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *combo)
{
  gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
      CLAMP (gconf_value_get_int (entry->value), 0, 1));
}

static void
ogmrip_pref_scaler_changed (GtkWidget *combo, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_SCALER, 
      gtk_combo_box_get_active (GTK_COMBO_BOX (combo)), NULL);
}

static void
ogmrip_pref_scaler_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *combo)
{
  gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
      CLAMP (gconf_value_get_int (entry->value), 0, 10));
}

static void
ogmrip_pref_trellis_toggled (GtkWidget *check, GConfClient *gconf)
{
  gconf_client_set_bool (gconf, OGMRIP_GCONF_TRELLIS, 
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)), NULL);
}

static void
ogmrip_pref_trellis_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *check)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
      gconf_value_get_bool (entry->value));
}

static void
ogmrip_pref_qpel_toggled (GtkWidget *check, GConfClient *gconf)
{
  gconf_client_set_bool (gconf, OGMRIP_GCONF_QPEL, 
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)), NULL);
}

static void
ogmrip_pref_qpel_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *check)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
      gconf_value_get_bool (entry->value));
}

static void
ogmrip_pref_turbo_toggled (GtkWidget *check, GConfClient *gconf)
{
  gconf_client_set_bool (gconf, OGMRIP_GCONF_TURBO, 
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)), NULL);
}

static void
ogmrip_pref_turbo_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *check)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
      gconf_value_get_bool (entry->value));
}

static void
ogmrip_pref_audio_format_toggled (GtkWidget *radio, GConfClient *gconf)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)))
    gconf_client_set_int (gconf, OGMRIP_GCONF_AUDIO,
        gtk_radio_button_get_active (GTK_RADIO_BUTTON (radio)), NULL);
}

static void
ogmrip_pref_audio_format_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *radio)
{
  gtk_radio_button_set_active (GTK_RADIO_BUTTON (radio),
      CLAMP (gconf_value_get_int (entry->value), 0, 3));
}

static void
ogmrip_pref_quality_changed (GtkWidget *spin, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_QUALITY, 
      gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)), NULL);
}

static void
ogmrip_pref_quality_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *spin)
{
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin),
      CLAMP (gconf_value_get_int (entry->value), 0, 10));
}

static void
ogmrip_pref_normalize_toggled (GtkWidget *check, GConfClient *gconf)
{
  gconf_client_set_bool (gconf, OGMRIP_GCONF_NORMALIZE, 
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)), NULL);
}

static void
ogmrip_pref_normalize_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *check)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
      gconf_value_get_bool (entry->value));
}

static void
ogmrip_pref_pref_audio_changed (GtkWidget *combo, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_PREF_AUDIO, 
      ogmrip_pref_get_active_lang (GTK_COMBO_BOX (combo)), NULL);
}

static void
ogmrip_pref_pref_audio_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *combo)
{
  ogmrip_pref_set_active_lang (GTK_COMBO_BOX (combo),
      MAX (gconf_value_get_int (entry->value), 0));
}

static void
ogmrip_pref_pref_subp_changed (GtkWidget *combo, GConfClient *gconf)
{
  gconf_client_set_int (gconf, OGMRIP_GCONF_PREF_SUBP, 
      ogmrip_pref_get_active_lang (GTK_COMBO_BOX (combo)), NULL);
}

static void
ogmrip_pref_pref_subp_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *combo)
{
  ogmrip_pref_set_active_lang (GTK_COMBO_BOX (combo),
      MAX (gconf_value_get_int (entry->value), 0));
}

static void
ogmrip_pref_spell_check_toggled (GtkWidget *check, GConfClient *gconf)
{
  gconf_client_set_bool (gconf, OGMRIP_GCONF_SPELL_CHECK, 
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)), NULL);
}

static void
ogmrip_pref_spell_check_notified (GConfClient *gconf, guint id, GConfEntry *entry, GtkWidget *check)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
      gconf_value_get_bool (entry->value));
}

GtkWidget *
ogmrip_pref_new (void)
{
  GConfClient *gconf;
  GtkWidget *dialog;
  GtkWidget *widget;
  GtkWidget *button;
  GladeXML *xml;
  gchar *str;
  guint i;

  xml = glade_xml_new (OGMRIP_DATA_DIR "/" OGMRIP_GLADE_FILE, NULL, NULL);
  if (!xml)
  {
    g_warning ("Could not find " OGMRIP_GLADE_FILE);
    return NULL;
  }

  gconf = gconf_client_get_default ();

  dialog = glade_xml_get_widget (xml, "pref-dialog");
  gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_PREFERENCES);
  g_signal_connect_after (dialog, "response", G_CALLBACK (gtk_widget_hide), NULL);

  g_signal_connect_swapped (dialog, "destroy", G_CALLBACK (g_object_unref), gconf);

  /*
   * General Tab
   */

  widget = glade_xml_get_widget (xml, "output-dir-entry");
  str = gconf_client_get_string_default (gconf, OGMRIP_GCONF_OUTPUT_DIR, OGMRIP_DEFAULT_OUTPUT_DIR);
  gtk_entry_set_text (GTK_ENTRY (widget), str);
  g_free (str);

  gconf_client_notify_add (gconf, OGMRIP_GCONF_OUTPUT_DIR, 
      (GConfClientNotifyFunc) ogmrip_pref_output_dir_notified, widget, NULL, NULL);

  button = glade_xml_get_widget (xml, "output-browse-button");
  g_signal_connect (button, "clicked", G_CALLBACK (ogmrip_pref_output_browse_clicked), gconf);

  widget = glade_xml_get_widget (xml, "tmp-dir-entry");
  str = gconf_client_get_string_default (gconf, OGMRIP_GCONF_TMP_DIR, OGMRIP_DEFAULT_TMP_DIR);
  gtk_entry_set_text (GTK_ENTRY (widget), str);
  ogmrip_set_tmp_dir (str);
  g_free (str);

  gconf_client_notify_add (gconf, OGMRIP_GCONF_OUTPUT_DIR, 
      (GConfClientNotifyFunc) ogmrip_pref_tmp_dir_notified, widget, NULL, NULL);

  button = glade_xml_get_widget (xml, "tmp-browse-button");
  g_signal_connect (button, "clicked", G_CALLBACK (ogmrip_pref_tmp_browse_clicked), gconf);

  widget = glade_xml_get_widget (xml, "filename-combo");
  gtk_combo_box_set_active (GTK_COMBO_BOX (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_FILENAME, OGMRIP_DEFAULT_FILENAME));
  g_signal_connect (widget, "changed", G_CALLBACK (ogmrip_pref_filename_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_FILENAME, 
      (GConfClientNotifyFunc) ogmrip_pref_filename_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "container-combo");
#if HAVE_MKV_SUPPORT
  gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Matroska Media (MKV)"));
#endif
  gtk_combo_box_set_active (GTK_COMBO_BOX (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_CONTAINER, OGMRIP_DEFAULT_CONTAINER));
  g_signal_connect (widget, "changed", G_CALLBACK (ogmrip_pref_container_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_CONTAINER, 
      (GConfClientNotifyFunc) ogmrip_pref_container_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "tnumber-spin");
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_TNUMBER, OGMRIP_DEFAULT_TNUMBER));
  g_signal_connect (widget, "value-changed", G_CALLBACK (ogmrip_pref_target_number_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_TNUMBER, 
      (GConfClientNotifyFunc) ogmrip_pref_target_number_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "tsize-spin");
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_TSIZE, OGMRIP_DEFAULT_TSIZE));
  g_signal_connect (widget, "value-changed", G_CALLBACK (ogmrip_pref_target_size_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_TSIZE, 
      (GConfClientNotifyFunc) ogmrip_pref_target_size_notified, widget, NULL, NULL);

  /*
   * Video Tab
   */

  widget = glade_xml_get_widget (xml, "xvid-radio");
  gtk_radio_button_set_active (GTK_RADIO_BUTTON (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_VIDEO, OGMRIP_DEFAULT_VIDEO));
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_video_format_toggled), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_VIDEO, 
      (GConfClientNotifyFunc) ogmrip_pref_video_format_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "lavc-radio");
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_video_format_toggled), gconf);

  widget = glade_xml_get_widget (xml, "x264-radio");
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_video_format_toggled), gconf);

#ifdef HAVE_X264_SUPPORT
  gtk_widget_set_sensitive (widget, TRUE);
#endif

  widget = glade_xml_get_widget (xml, "theora-radio");
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_video_format_toggled), gconf);

#ifdef HAVE_THEORA_SUPPORT
  gtk_widget_set_sensitive (widget, TRUE);
#endif

  widget = glade_xml_get_widget (xml, "passes-combo");
  gtk_combo_box_set_active (GTK_COMBO_BOX (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_PASSES, OGMRIP_DEFAULT_PASSES));
  g_signal_connect (widget, "changed", G_CALLBACK (ogmrip_pref_passes_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_PASSES, 
      (GConfClientNotifyFunc) ogmrip_pref_passes_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "scaler-combo");
  gtk_combo_box_set_active (GTK_COMBO_BOX (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_SCALER, OGMRIP_DEFAULT_SCALER));
  g_signal_connect (widget, "changed", G_CALLBACK (ogmrip_pref_scaler_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_SCALER, 
      (GConfClientNotifyFunc) ogmrip_pref_scaler_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "trellis-check");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
      gconf_client_get_bool_default (gconf, OGMRIP_GCONF_TRELLIS, OGMRIP_DEFAULT_TRELLIS));
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_trellis_toggled), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_TRELLIS, 
      (GConfClientNotifyFunc) ogmrip_pref_trellis_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "qpel-check");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
      gconf_client_get_bool_default (gconf, OGMRIP_GCONF_QPEL, OGMRIP_DEFAULT_QPEL));
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_qpel_toggled), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_QPEL, 
      (GConfClientNotifyFunc) ogmrip_pref_qpel_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "turbo-check");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
      gconf_client_get_bool_default (gconf, OGMRIP_GCONF_TURBO, OGMRIP_DEFAULT_TURBO));
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_turbo_toggled), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_TURBO, 
      (GConfClientNotifyFunc) ogmrip_pref_turbo_notified, widget, NULL, NULL);

  /*
   * Audio Tab
   */

  widget = glade_xml_get_widget (xml, "pref-audio-combo");
  for (i = 2; i < ogmdvd_nlanguages; i++)
  {
    str = g_strdup_printf ("%s (%s)", ogmdvd_languages[i][OGMDVD_LANGUAGE_NAME], 
        ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1]);
    gtk_combo_box_append_text (GTK_COMBO_BOX (widget), str);
    g_free (str);
  }
  ogmrip_pref_set_active_lang (GTK_COMBO_BOX (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_PREF_AUDIO, OGMRIP_DEFAULT_PREF_AUDIO));
  g_signal_connect (widget, "changed", G_CALLBACK (ogmrip_pref_pref_audio_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_PREF_AUDIO, 
      (GConfClientNotifyFunc) ogmrip_pref_pref_audio_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "vorbis-radio");
  gtk_radio_button_set_active (GTK_RADIO_BUTTON (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_AUDIO, OGMRIP_DEFAULT_AUDIO));
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_audio_format_toggled), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_AUDIO, 
      (GConfClientNotifyFunc) ogmrip_pref_audio_format_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "mp3-radio");
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_audio_format_toggled), gconf);

  widget = glade_xml_get_widget (xml, "wave-radio");
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_audio_format_toggled), gconf);

  widget = glade_xml_get_widget (xml, "ac3-radio");
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_audio_format_toggled), gconf);

  widget = glade_xml_get_widget (xml, "quality-spin");
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_QUALITY, OGMRIP_DEFAULT_QUALITY));
  g_signal_connect (widget, "value-changed", G_CALLBACK (ogmrip_pref_quality_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_QUALITY, 
      (GConfClientNotifyFunc) ogmrip_pref_quality_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "normalize-check");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
      gconf_client_get_bool_default (gconf, OGMRIP_GCONF_NORMALIZE, OGMRIP_DEFAULT_NORMALIZE));
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_normalize_toggled), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_NORMALIZE, 
      (GConfClientNotifyFunc) ogmrip_pref_normalize_notified, widget, NULL, NULL);

  /*
   * Subtitles Tab
   */

  widget = glade_xml_get_widget (xml, "pref-subp-combo");
  for (i = 2; i < ogmdvd_nlanguages; i++)
  {
    str = g_strdup_printf ("%s (%s)", ogmdvd_languages[i][OGMDVD_LANGUAGE_NAME], 
        ogmdvd_languages[i][OGMDVD_LANGUAGE_ISO639_1]);
    gtk_combo_box_append_text (GTK_COMBO_BOX (widget), str);
    g_free (str);
  }
  ogmrip_pref_set_active_lang (GTK_COMBO_BOX (widget),
      gconf_client_get_int_default (gconf, OGMRIP_GCONF_PREF_SUBP, OGMRIP_DEFAULT_PREF_SUBP));
  g_signal_connect (widget, "changed", G_CALLBACK (ogmrip_pref_pref_subp_changed), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_PREF_SUBP, 
      (GConfClientNotifyFunc) ogmrip_pref_pref_subp_notified, widget, NULL, NULL);

  widget = glade_xml_get_widget (xml, "spell-check");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
      gconf_client_get_bool_default (gconf, OGMRIP_GCONF_SPELL_CHECK, OGMRIP_DEFAULT_SPELL_CHECK));
  g_signal_connect (widget, "toggled", G_CALLBACK (ogmrip_pref_spell_check_toggled), gconf);
  gconf_client_notify_add (gconf, OGMRIP_GCONF_SPELL_CHECK, 
      (GConfClientNotifyFunc) ogmrip_pref_spell_check_notified, widget, NULL, NULL);

#if defined(HAVE_SRT_SUPPORT) && defined(HAVE_ENCHANT_SUPPORT)
  gtk_widget_set_sensitive (widget, TRUE);
#endif

  g_object_unref (xml);

  return dialog;
}

